import { Component, ViewChild, Input } from '@angular/core';
import { Configuration } from './config';
import { REGISTRY_CONFIG_HTML } from './registry-config.component.html';
import { ConfigurationService, SystemInfoService } from '../service/index';
import { toPromise, compareValue, isEmptyObject, clone } from '../utils';
import { ErrorHandler } from '../error-handler';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { TranslateService } from '@ngx-translate/core';
var RegistryConfigComponent = (function () {
    function RegistryConfigComponent(configService, errorHandler, translate, systemInfoService) {
        this.configService = configService;
        this.errorHandler = errorHandler;
        this.translate = translate;
        this.systemInfoService = systemInfoService;
        this.config = new Configuration();
        this.onGoing = false;
        this.hasAdminRole = false;
    }
    Object.defineProperty(RegistryConfigComponent.prototype, "shouldDisable", {
        get: function () {
            return !this.isValid() || !this.hasChanges() || this.onGoing;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RegistryConfigComponent.prototype, "hasCAFile", {
        get: function () {
            return this.systemInfo && this.systemInfo.has_ca_root;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RegistryConfigComponent.prototype, "withClair", {
        get: function () {
            return this.systemInfo && this.systemInfo.with_clair;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RegistryConfigComponent.prototype, "withAdmiral", {
        get: function () {
            return this.systemInfo && this.systemInfo.with_admiral;
        },
        enumerable: true,
        configurable: true
    });
    RegistryConfigComponent.prototype.ngOnInit = function () {
        this.loadSystemInfo();
        //Initialize
        this.load();
    };
    RegistryConfigComponent.prototype.isValid = function () {
        return this.systemSettings &&
            this.systemSettings.isValid &&
            this.vulnerabilityCfg &&
            this.vulnerabilityCfg.isValid;
    };
    RegistryConfigComponent.prototype.hasChanges = function () {
        return !isEmptyObject(this.getChanges());
    };
    //Get system info
    RegistryConfigComponent.prototype.loadSystemInfo = function () {
        var _this = this;
        toPromise(this.systemInfoService.getSystemInfo())
            .then(function (info) { return _this.systemInfo = info; })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    //Load configurations
    RegistryConfigComponent.prototype.load = function () {
        var _this = this;
        this.onGoing = true;
        toPromise(this.configService.getConfigurations())
            .then(function (config) {
            _this.configCopy = clone(config);
            _this.config = config;
            _this.onGoing = false;
        })
            .catch(function (error) {
            _this.errorHandler.error(error);
            _this.onGoing = false;
        });
    };
    //Save configuration changes
    RegistryConfigComponent.prototype.save = function () {
        var _this = this;
        var changes = this.getChanges();
        if (isEmptyObject(changes)) {
            //Guard code, do nothing
            return;
        }
        this.onGoing = true;
        toPromise(this.configService.saveConfigurations(changes))
            .then(function () {
            _this.onGoing = false;
            _this.translate.get("CONFIG.SAVE_SUCCESS").subscribe(function (res) {
                _this.errorHandler.info(res);
            });
            //Reload to fetch all the updates
            _this.load();
            //Reload all system info 
            //this.loadSystemInfo();
        })
            .catch(function (error) {
            _this.onGoing = false;
            _this.errorHandler.error(error);
        });
    };
    //Cancel the changes if have
    RegistryConfigComponent.prototype.cancel = function () {
        var msg = new ConfirmationMessage("CONFIG.CONFIRM_TITLE", "CONFIG.CONFIRM_SUMMARY", "", {}, 9 /* CONFIG */);
        this.confirmationDlg.open(msg);
    };
    //Confirm cancel
    RegistryConfigComponent.prototype.confirmCancel = function (ack) {
        if (ack && ack.source === 9 /* CONFIG */ &&
            ack.state === 1 /* CONFIRMED */) {
            this.reset();
        }
    };
    RegistryConfigComponent.prototype.reset = function () {
        //Reset to the values of copy
        var changes = this.getChanges();
        for (var prop in changes) {
            this.config[prop] = clone(this.configCopy[prop]);
        }
    };
    RegistryConfigComponent.prototype.getChanges = function () {
        var changes = {};
        if (!this.config || !this.configCopy) {
            return changes;
        }
        for (var prop in this.config) {
            var field = this.configCopy[prop];
            if (field && field.editable) {
                if (!compareValue(field.value, this.config[prop].value)) {
                    changes[prop] = this.config[prop].value;
                    //Number 
                    if (typeof field.value === "number") {
                        changes[prop] = +changes[prop];
                    }
                    //Trim string value
                    if (typeof field.value === "string") {
                        changes[prop] = ('' + changes[prop]).trim();
                    }
                }
            }
        }
        return changes;
    };
    return RegistryConfigComponent;
}());
export { RegistryConfigComponent };
RegistryConfigComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-registry-config',
                template: REGISTRY_CONFIG_HTML
            },] },
];
/** @nocollapse */
RegistryConfigComponent.ctorParameters = function () { return [
    { type: ConfigurationService, },
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: SystemInfoService, },
]; };
RegistryConfigComponent.propDecorators = {
    'hasAdminRole': [{ type: Input },],
    'systemSettings': [{ type: ViewChild, args: ["systemSettings",] },],
    'vulnerabilityCfg': [{ type: ViewChild, args: ["vulnerabilityConfig",] },],
    'confirmationDlg': [{ type: ViewChild, args: ["cfgConfirmationDialog",] },],
};
//# sourceMappingURL=registry-config.component.js.map