// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, Output, EventEmitter, ViewChild, ChangeDetectorRef } from '@angular/core';
import { EndpointService } from '../service/endpoint.service';
import { ErrorHandler } from '../error-handler/index';
import { InlineAlertComponent } from '../inline-alert/inline-alert.component';
import { TranslateService } from '@ngx-translate/core';
import { CREATE_EDIT_ENDPOINT_STYLE } from './create-edit-endpoint.component.css';
import { CREATE_EDIT_ENDPOINT_TEMPLATE } from './create-edit-endpoint.component.html';
import { toPromise, clone, compareValue, isEmptyObject } from '../utils';
var FAKE_PASSWORD = 'rjGcfuRu';
var CreateEditEndpointComponent = (function () {
    function CreateEditEndpointComponent(endpointService, errorHandler, translateService, ref) {
        this.endpointService = endpointService;
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.ref = ref;
        this.staticBackdrop = true;
        this.closable = false;
        this.target = this.initEndpoint();
        this.reload = new EventEmitter();
    }
    Object.defineProperty(CreateEditEndpointComponent.prototype, "isValid", {
        get: function () {
            return !this.testOngoing &&
                !this.onGoing &&
                this.targetForm &&
                this.targetForm.valid &&
                this.editable &&
                !compareValue(this.target, this.initVal);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CreateEditEndpointComponent.prototype, "inProgress", {
        get: function () {
            return this.onGoing || this.testOngoing;
        },
        enumerable: true,
        configurable: true
    });
    CreateEditEndpointComponent.prototype.setInsecureValue = function ($event) {
        this.target.insecure = !$event;
    };
    CreateEditEndpointComponent.prototype.ngOnDestroy = function () {
        if (this.valueChangesSub) {
            this.valueChangesSub.unsubscribe();
        }
    };
    CreateEditEndpointComponent.prototype.initEndpoint = function () {
        return {
            endpoint: "",
            name: "",
            username: "",
            password: "",
            insecure: false,
            type: 0
        };
    };
    CreateEditEndpointComponent.prototype.open = function () {
        this.createEditDestinationOpened = true;
    };
    CreateEditEndpointComponent.prototype.close = function () {
        this.createEditDestinationOpened = false;
    };
    CreateEditEndpointComponent.prototype.reset = function () {
        //Reset status variables
        this.testOngoing = false;
        this.onGoing = false;
        //Reset data
        this.target = this.initEndpoint();
        this.initVal = this.initEndpoint();
        this.formValues = null;
        this.endpointId = '';
        this.inlineAlert.close();
    };
    //Forcely refresh the view
    CreateEditEndpointComponent.prototype.forceRefreshView = function (duration) {
        var _this = this;
        //Reset timer
        if (this.timerHandler) {
            clearInterval(this.timerHandler);
        }
        this.timerHandler = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () {
            if (_this.timerHandler) {
                clearInterval(_this.timerHandler);
                _this.timerHandler = null;
            }
        }, duration);
    };
    CreateEditEndpointComponent.prototype.openCreateEditTarget = function (editable, targetId) {
        var _this = this;
        this.editable = editable;
        //reset
        this.reset();
        if (targetId) {
            this.endpointId = targetId;
            this.translateService.get('DESTINATION.TITLE_EDIT').subscribe(function (res) { return _this.modalTitle = res; });
            toPromise(this.endpointService
                .getEndpoint(targetId))
                .then(function (target) {
                _this.target = target;
                //Keep data cache
                _this.initVal = clone(target);
                _this.initVal.password = FAKE_PASSWORD;
                _this.target.password = FAKE_PASSWORD;
                //Open the modal now
                _this.open();
                _this.forceRefreshView(2000);
            })
                .catch(function (error) { return _this.errorHandler.error(error); });
        }
        else {
            this.endpointId = '';
            this.translateService.get('DESTINATION.TITLE_ADD').subscribe(function (res) { return _this.modalTitle = res; });
            //Directly open the modal
            this.open();
        }
    };
    CreateEditEndpointComponent.prototype.testConnection = function () {
        var _this = this;
        var payload = this.initEndpoint();
        if (!this.endpointId) {
            payload.endpoint = this.target.endpoint;
            payload.username = this.target.username;
            payload.password = this.target.password;
            payload.insecure = this.target.insecure;
        }
        else {
            var changes_1 = this.getChanges();
            for (var prop in payload) {
                delete payload[prop];
            }
            payload.id = this.target.id;
            if (!isEmptyObject(changes_1)) {
                var changekeys = Object.keys(this.getChanges());
                changekeys.forEach(function (key) {
                    payload[key] = changes_1[key];
                });
            }
        }
        this.testOngoing = true;
        toPromise(this.endpointService
            .pingEndpoint(payload))
            .then(function (response) {
            _this.inlineAlert.showInlineSuccess({ message: "DESTINATION.TEST_CONNECTION_SUCCESS" });
            _this.forceRefreshView(2000);
            _this.testOngoing = false;
        }).catch(function (error) {
            _this.inlineAlert.showInlineError('DESTINATION.TEST_CONNECTION_FAILURE');
            _this.forceRefreshView(2000);
            _this.testOngoing = false;
        });
    };
    CreateEditEndpointComponent.prototype.onSubmit = function () {
        if (this.endpointId) {
            this.updateEndpoint();
        }
        else {
            this.addEndpoint();
        }
    };
    CreateEditEndpointComponent.prototype.addEndpoint = function () {
        var _this = this;
        if (this.onGoing) {
            return; //Avoid duplicated submitting
        }
        this.onGoing = true;
        toPromise(this.endpointService
            .createEndpoint(this.target))
            .then(function (response) {
            _this.translateService.get('DESTINATION.CREATED_SUCCESS')
                .subscribe(function (res) { return _this.errorHandler.info(res); });
            _this.reload.emit(true);
            _this.onGoing = false;
            _this.close();
            _this.forceRefreshView(2000);
        }).catch(function (error) {
            _this.onGoing = false;
            var errorMessageKey = _this.handleErrorMessageKey(error.status);
            _this.translateService
                .get(errorMessageKey)
                .subscribe(function (res) {
                _this.inlineAlert.showInlineError(res);
            });
            _this.forceRefreshView(2000);
        });
    };
    CreateEditEndpointComponent.prototype.updateEndpoint = function () {
        var _this = this;
        if (this.onGoing) {
            return; //Avoid duplicated submitting
        }
        var payload = this.initEndpoint();
        for (var prop in payload) {
            delete payload[prop];
        }
        var changes = this.getChanges();
        if (isEmptyObject(changes)) {
            return;
        }
        var changekeys = Object.keys(changes);
        changekeys.forEach(function (key) {
            payload[key] = changes[key];
        });
        if (!this.target.id) {
            return;
        }
        this.onGoing = true;
        toPromise(this.endpointService
            .updateEndpoint(this.target.id, payload))
            .then(function (response) {
            _this.translateService.get('DESTINATION.UPDATED_SUCCESS')
                .subscribe(function (res) { return _this.errorHandler.info(res); });
            _this.reload.emit(true);
            _this.close();
            _this.onGoing = false;
            _this.forceRefreshView(2000);
        })
            .catch(function (error) {
            var errorMessageKey = _this.handleErrorMessageKey(error.status);
            _this.translateService
                .get(errorMessageKey)
                .subscribe(function (res) {
                _this.inlineAlert.showInlineError(res);
            });
            _this.onGoing = false;
            _this.forceRefreshView(2000);
        });
    };
    CreateEditEndpointComponent.prototype.handleErrorMessageKey = function (status) {
        switch (status) {
            case 409:
                return 'DESTINATION.CONFLICT_NAME';
            case 400:
                return 'DESTINATION.INVALID_NAME';
            default:
                return 'UNKNOWN_ERROR';
        }
    };
    CreateEditEndpointComponent.prototype.onCancel = function () {
        var changes = this.getChanges();
        if (!isEmptyObject(changes)) {
            this.inlineAlert.showInlineConfirmation({ message: 'ALERT.FORM_CHANGE_CONFIRMATION' });
        }
        else {
            this.close();
            if (this.targetForm) {
                this.targetForm.reset();
            }
        }
    };
    CreateEditEndpointComponent.prototype.confirmCancel = function (confirmed) {
        this.inlineAlert.close();
        this.close();
    };
    CreateEditEndpointComponent.prototype.ngAfterViewChecked = function () {
        var _this = this;
        if (this.targetForm != this.currentForm) {
            this.targetForm = this.currentForm;
            if (this.targetForm) {
                this.valueChangesSub = this.targetForm.valueChanges.subscribe(function (data) {
                    if (data) {
                        //To avoid invalid change publish events
                        var keyNumber = 0;
                        for (var key in data) {
                            //Empty string "" is accepted
                            if (data[key] !== null) {
                                keyNumber++;
                            }
                        }
                        if (keyNumber !== 5) {
                            return;
                        }
                        if (!compareValue(_this.formValues, data)) {
                            _this.formValues = data;
                            _this.inlineAlert.close();
                        }
                    }
                });
            }
        }
    };
    CreateEditEndpointComponent.prototype.getChanges = function () {
        var changes = {};
        if (!this.target || !this.initVal) {
            return changes;
        }
        for (var prop in this.target) {
            var field = this.initVal[prop];
            if (!compareValue(field, this.target[prop])) {
                changes[prop] = this.target[prop];
                //Number
                if (typeof field === "number") {
                    changes[prop] = +changes[prop];
                }
                //Trim string value
                if (typeof field === "string") {
                    changes[prop] = ('' + changes[prop]).trim();
                }
            }
        }
        return changes;
    };
    return CreateEditEndpointComponent;
}());
export { CreateEditEndpointComponent };
CreateEditEndpointComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-create-edit-endpoint',
                template: CREATE_EDIT_ENDPOINT_TEMPLATE,
                styles: [CREATE_EDIT_ENDPOINT_STYLE]
            },] },
];
/** @nocollapse */
CreateEditEndpointComponent.ctorParameters = function () { return [
    { type: EndpointService, },
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
]; };
CreateEditEndpointComponent.propDecorators = {
    'currentForm': [{ type: ViewChild, args: ['targetForm',] },],
    'inlineAlert': [{ type: ViewChild, args: [InlineAlertComponent,] },],
    'reload': [{ type: Output },],
};
//# sourceMappingURL=create-edit-endpoint.component.js.map