// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, ChangeDetectorRef, Input, EventEmitter, Output } from '@angular/core';
import { Subject } from "rxjs/Subject";
import { FormBuilder, Validators } from "@angular/forms";
import { compareValue, isEmptyObject, toPromise } from "../utils";
import { InlineAlertComponent } from '../inline-alert/inline-alert.component';
import { ReplicationService } from "../service/replication.service";
import { CREATE_EDIT_RULE_TEMPLATE } from "./create-edit-rule.component.html";
import { CREATE_EDIT_RULE_STYLE } from "./create-edit-rule.component.css";
import { ErrorHandler } from "../error-handler/error-handler";
import { TranslateService } from "@ngx-translate/core";
import { EndpointService } from "../service/endpoint.service";
import { ProjectService } from "../service/project.service";
var ONE_HOUR_SECONDS = 3600;
var ONE_DAY_SECONDS = 24 * ONE_HOUR_SECONDS;
var CreateEditRuleComponent = (function () {
    function CreateEditRuleComponent(fb, repService, endpointService, errorHandler, proService, translateService, ref) {
        this.fb = fb;
        this.repService = repService;
        this.endpointService = endpointService;
        this.errorHandler = errorHandler;
        this.proService = proService;
        this.translateService = translateService;
        this.ref = ref;
        this._localTime = new Date();
        this.targetList = [];
        this.projectList = [];
        this.selectedProjectList = [];
        this.isFilterHide = false;
        this.isImmediate = false;
        this.noProjectInfo = "";
        this.noEndpointInfo = "";
        this.noSelectedProject = true;
        this.noSelectedEndpoint = true;
        this.filterCount = 0;
        this.triggerNames = ['Manual', 'Immediate', 'Scheduled'];
        this.scheduleNames = ['Daily', 'Weekly'];
        this.weekly = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'];
        this.filterSelect = ['repository', 'tag'];
        this.ruleNameTooltip = 'TOOLTIP.EMPTY';
        this.headerTitle = 'REPLICATION.ADD_POLICY';
        this.filterListData = [];
        this.inProgress = false;
        this.inNameChecking = false;
        this.isRuleNameExist = false;
        this.nameChecker = new Subject();
        this.proNameChecker = new Subject();
        this.firstClick = 0;
        this.goToRegistry = new EventEmitter();
        this.reload = new EventEmitter();
        this.emptyProject = {
            project_id: -1,
            name: '',
        };
        this.emptyEndpoint = {
            id: -1,
            endpoint: '',
            name: '',
            username: '',
            password: '',
            insecure: true,
            type: 0,
        };
        this.createForm();
    }
    CreateEditRuleComponent.prototype.baseFilterData = function (name, option, state) {
        return {
            name: name,
            options: option,
            state: state,
            isValid: true
        };
    };
    CreateEditRuleComponent.prototype.ngOnInit = function () {
        var _this = this;
        toPromise(this.endpointService
            .getEndpoints())
            .then(function (targets) {
            _this.targetList = targets || [];
        }).catch(function (error) { return _this.errorHandler.error(error); });
        if (!this.projectId) {
            toPromise(this.proService.listProjects("", undefined))
                .then(function (targets) {
                _this.projectList = targets || [];
            }).catch(function (error) { return _this.errorHandler.error(error); });
        }
        this.nameChecker.debounceTime(500).distinctUntilChanged().subscribe(function (ruleName) {
            _this.isRuleNameExist = false;
            _this.inNameChecking = true;
            toPromise(_this.repService.getReplicationRules(0, ruleName))
                .then(function (response) {
                if (response.some(function (rule) { return rule.name === ruleName; })) {
                    _this.ruleNameTooltip = 'TOOLTIP.RULE_USER_EXISTING';
                    _this.isRuleNameExist = true;
                }
                _this.inNameChecking = false;
            }).catch(function () {
                _this.inNameChecking = false;
            });
        });
        this.proNameChecker
            .debounceTime(500)
            .distinctUntilChanged()
            .subscribe(function (name) {
            _this.noProjectInfo = '';
            _this.selectedProjectList = [];
            toPromise(_this.proService.listProjects(name, undefined)).then(function (res) {
                if (res) {
                    _this.selectedProjectList = res.slice(0, 10);
                    // if input value exit in project list
                    var pro = res.find(function (data) { return data.name === name; });
                    if (!pro) {
                        _this.noProjectInfo = 'REPLICATION.NO_PROJECT_INFO';
                        _this.noSelectedProject = true;
                    }
                    else {
                        _this.noProjectInfo = '';
                        _this.noSelectedProject = false;
                        _this.setProject([pro]);
                    }
                }
                else {
                    _this.noProjectInfo = 'REPLICATION.NO_PROJECT_INFO';
                    _this.noSelectedProject = true;
                }
            }).catch(function (error) {
                _this.errorHandler.error(error);
                _this.noProjectInfo = 'REPLICATION.NO_PROJECT_INFO';
                _this.noSelectedProject = true;
            });
        });
    };
    CreateEditRuleComponent.prototype.ngOnDestroy = function () {
        if (this.confirmSub) {
            this.confirmSub.unsubscribe();
        }
        if (this.nameChecker) {
            this.nameChecker.unsubscribe();
        }
        if (this.proNameChecker) {
            this.proNameChecker.unsubscribe();
        }
    };
    Object.defineProperty(CreateEditRuleComponent.prototype, "isValid", {
        get: function () {
            return !(this.isRuleNameExist || this.noSelectedProject || this.noSelectedEndpoint || this.inProgress);
        },
        enumerable: true,
        configurable: true
    });
    CreateEditRuleComponent.prototype.createForm = function () {
        this.ruleForm = this.fb.group({
            name: ['', Validators.required],
            description: '',
            projects: this.fb.array([]),
            targets: this.fb.array([]),
            trigger: this.fb.group({
                kind: this.triggerNames[0],
                schedule_param: this.fb.group({
                    type: this.scheduleNames[0],
                    weekday: 1,
                    offtime: '08:00'
                }),
            }),
            filters: this.fb.array([]),
            replicate_existing_image_now: true,
            replicate_deletion: false
        });
    };
    CreateEditRuleComponent.prototype.initForm = function () {
        this.ruleForm.reset({
            name: '',
            description: '',
            trigger: { kind: this.triggerNames[0], schedule_param: {
                    type: this.scheduleNames[0],
                    weekday: 1,
                    offtime: '08:00'
                } },
            replicate_existing_image_now: true,
            replicate_deletion: false
        });
        this.setProject([this.emptyProject]);
        this.setTarget([this.emptyEndpoint]);
        this.setFilter([]);
        this.copyUpdateForm = Object.assign({}, this.ruleForm.value);
    };
    CreateEditRuleComponent.prototype.updateForm = function (rule) {
        var _this = this;
        rule.trigger = this.updateTrigger(rule.trigger);
        this.ruleForm.reset({
            name: rule.name,
            description: rule.description,
            trigger: rule.trigger,
            replicate_existing_image_now: rule.replicate_existing_image_now,
            replicate_deletion: rule.replicate_deletion
        });
        this.setProject(rule.projects);
        this.noSelectedProject = false;
        this.setTarget(rule.targets);
        this.noSelectedEndpoint = false;
        if (rule.filters) {
            this.setFilter(rule.filters);
            this.updateFilter(rule.filters);
        }
        // Force refresh view
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 2000);
    };
    Object.defineProperty(CreateEditRuleComponent.prototype, "projects", {
        get: function () {
            return this.ruleForm.get('projects');
        },
        enumerable: true,
        configurable: true
    });
    CreateEditRuleComponent.prototype.setProject = function (projects) {
        var _this = this;
        var projectFGs = projects.map(function (project) { return _this.fb.group(project); });
        var projectFormArray = this.fb.array(projectFGs);
        this.ruleForm.setControl('projects', projectFormArray);
    };
    Object.defineProperty(CreateEditRuleComponent.prototype, "filters", {
        get: function () {
            return this.ruleForm.get('filters');
        },
        enumerable: true,
        configurable: true
    });
    CreateEditRuleComponent.prototype.setFilter = function (filters) {
        var _this = this;
        var filterFGs = filters.map(function (filter) { return _this.fb.group(filter); });
        var filterFormArray = this.fb.array(filterFGs);
        this.ruleForm.setControl('filters', filterFormArray);
    };
    Object.defineProperty(CreateEditRuleComponent.prototype, "targets", {
        get: function () {
            return this.ruleForm.get('targets');
        },
        enumerable: true,
        configurable: true
    });
    CreateEditRuleComponent.prototype.setTarget = function (targets) {
        var _this = this;
        var targetFGs = targets.map(function (target) { return _this.fb.group(target); });
        var targetFormArray = this.fb.array(targetFGs);
        this.ruleForm.setControl('targets', targetFormArray);
    };
    CreateEditRuleComponent.prototype.initFilter = function (name) {
        return this.fb.group({
            kind: name,
            pattern: ['', Validators.required]
        });
    };
    CreateEditRuleComponent.prototype.filterChange = function ($event) {
        if ($event && $event.target['value']) {
            var id_1 = $event.target.id;
            var name_1 = $event.target.name;
            var value_1 = $event.target['value'];
            this.filterListData.forEach(function (data, index) {
                if (index === +id_1) {
                    data.name = $event.target.name = value_1;
                }
                else {
                    data.options.splice(data.options.indexOf(value_1), 1);
                }
                if (data.options.indexOf(name_1) === -1) {
                    data.options.push(name_1);
                }
            });
        }
    };
    CreateEditRuleComponent.prototype.targetChange = function ($event) {
        if ($event && $event.target) {
            if ($event.target['value'] === '-1') {
                this.noSelectedEndpoint = true;
                return;
            }
            var selecedTarget = this.targetList.find(function (target) { return target.id === +$event.target['value']; });
            this.setTarget([selecedTarget]);
            this.noSelectedEndpoint = false;
        }
    };
    // Handle the form validation
    CreateEditRuleComponent.prototype.handleValidation = function () {
        var cont = this.ruleForm.controls["projects"];
        if (cont && cont.valid) {
            this.proNameChecker.next(cont.value[0].name);
        }
    };
    CreateEditRuleComponent.prototype.focusClear = function ($event) {
        if (this.policyId < 0 && this.firstClick === 0) {
            if ($event && $event.target && $event.target['value']) {
                $event.target['value'] = '';
            }
            this.firstClick++;
        }
    };
    CreateEditRuleComponent.prototype.leaveInput = function () {
        this.selectedProjectList = [];
    };
    CreateEditRuleComponent.prototype.selectedProjectName = function (projectName) {
        this.noSelectedProject = false;
        var pro = this.selectedProjectList.find(function (data) { return data.name === projectName; });
        this.setProject([pro]);
        this.selectedProjectList = [];
        this.noProjectInfo = "";
    };
    CreateEditRuleComponent.prototype.selectedProject = function (project) {
        if (!project) {
            this.noSelectedProject = true;
        }
        else {
            this.noSelectedProject = false;
            this.setProject([project]);
        }
    };
    CreateEditRuleComponent.prototype.addNewFilter = function () {
        if (this.filterCount === 0) {
            this.filterListData.push(this.baseFilterData(this.filterSelect[0], this.filterSelect.slice(), true));
            this.filters.push(this.initFilter(this.filterSelect[0]));
        }
        else {
            var nameArr_1 = this.filterSelect.slice();
            this.filterListData.forEach(function (data) {
                nameArr_1.splice(nameArr_1.indexOf(data.name), 1);
            });
            // when add a new filter,the filterListData should change the options
            this.filterListData.filter(function (data) {
                data.options.splice(data.options.indexOf(nameArr_1[0]), 1);
            });
            this.filterListData.push(this.baseFilterData(nameArr_1[0], nameArr_1, true));
            this.filters.push(this.initFilter(nameArr_1[0]));
        }
        this.filterCount += 1;
        if (this.filterCount >= this.filterSelect.length) {
            this.isFilterHide = true;
        }
    };
    // delete a filter
    CreateEditRuleComponent.prototype.deleteFilter = function (i) {
        if (i || i === 0) {
            var delfilter = this.filterListData.splice(i, 1)[0];
            if (this.filterCount === this.filterSelect.length) {
                this.isFilterHide = false;
            }
            this.filterCount -= 1;
            if (this.filterListData.length) {
                var optionVal_1 = delfilter.name;
                this.filterListData.filter(function (data) {
                    if (data.options.indexOf(optionVal_1) === -1) {
                        data.options.push(optionVal_1);
                    }
                });
            }
            var control = this.ruleForm.controls['filters'];
            control.removeAt(i);
        }
    };
    CreateEditRuleComponent.prototype.selectTrigger = function ($event) {
        if ($event && $event.target && $event.target['value']) {
            var val = $event.target['value'];
            if (val === this.triggerNames[2]) {
                this.isScheduleOpt = true;
                this.isImmediate = false;
            }
            if (val === this.triggerNames[1]) {
                this.isScheduleOpt = false;
                this.isImmediate = true;
            }
            if (val === this.triggerNames[0]) {
                this.isScheduleOpt = false;
                this.isImmediate = false;
            }
        }
    };
    // Replication Schedule select value exchange
    CreateEditRuleComponent.prototype.selectSchedule = function ($event) {
        if ($event && $event.target && $event.target['value']) {
            switch ($event.target['value']) {
                case this.scheduleNames[1]:
                    this.weeklySchedule = true;
                    this.ruleForm.patchValue({
                        trigger: {
                            schedule_param: {
                                weekday: 1,
                            }
                        }
                    });
                    break;
                case this.scheduleNames[0]:
                    this.weeklySchedule = false;
                    break;
            }
        }
    };
    CreateEditRuleComponent.prototype.checkRuleName = function () {
        var ruleName = this.ruleForm.controls['name'].value;
        if (ruleName) {
            this.nameChecker.next(ruleName);
        }
        else {
            this.ruleNameTooltip = 'TOOLTIP.EMPTY';
        }
    };
    CreateEditRuleComponent.prototype.updateFilter = function (filters) {
        var _this = this;
        var opt = this.filterSelect.slice();
        filters.forEach(function (filter) {
            opt.splice(opt.indexOf(filter.kind), 1);
        });
        filters.forEach(function (filter) {
            var option = opt.slice();
            option.unshift(filter.kind);
            _this.filterListData.push(_this.baseFilterData(filter.kind, option, true));
        });
        this.filterCount = filters.length;
        if (filters.length === this.filterSelect.length) {
            this.isFilterHide = true;
        }
    };
    CreateEditRuleComponent.prototype.updateTrigger = function (trigger) {
        if (trigger['schedule_param']) {
            this.isScheduleOpt = true;
            this.isImmediate = false;
            trigger['schedule_param']['offtime'] = this.getOfftime(trigger['schedule_param']['offtime']);
            if (trigger['schedule_param']['weekday']) {
                this.weeklySchedule = true;
            }
            else {
                // set default
                trigger['schedule_param']['weekday'] = 1;
            }
        }
        else {
            if (trigger['kind'] === this.triggerNames[0]) {
                this.isImmediate = false;
            }
            if (trigger['kind'] === this.triggerNames[1]) {
                this.isImmediate = true;
            }
            trigger['schedule_param'] = { type: this.scheduleNames[0],
                weekday: this.weekly[0],
                offtime: '08:00' };
        }
        return trigger;
    };
    CreateEditRuleComponent.prototype.setTriggerVaule = function (trigger) {
        if (!this.isScheduleOpt) {
            delete trigger['schedule_param'];
            return trigger;
        }
        else {
            if (!this.weeklySchedule) {
                delete trigger['schedule_param']['weekday'];
            }
            else {
                trigger['schedule_param']['weekday'] = +trigger['schedule_param']['weekday'];
            }
            trigger['schedule_param']['offtime'] = this.setOfftime(trigger['schedule_param']['offtime']);
            return trigger;
        }
    };
    CreateEditRuleComponent.prototype.hasFormChange = function () {
        return !isEmptyObject(this.getChanges());
    };
    CreateEditRuleComponent.prototype.onSubmit = function () {
        var _this = this;
        // add new Replication rule
        this.inProgress = true;
        var copyRuleForm = this.ruleForm.value;
        copyRuleForm.trigger = this.setTriggerVaule(copyRuleForm.trigger);
        if (this.policyId < 0) {
            this.repService.createReplicationRule(copyRuleForm)
                .then(function () {
                _this.translateService.get('REPLICATION.CREATED_SUCCESS')
                    .subscribe(function (res) { return _this.errorHandler.info(res); });
                _this.inProgress = false;
                _this.reload.emit(true);
                _this.close();
            }).catch(function (error) {
                _this.inProgress = false;
                _this.inlineAlert.showInlineError(error);
            });
        }
        else {
            this.repService.updateReplicationRule(this.policyId, this.ruleForm.value)
                .then(function () {
                _this.translateService.get('REPLICATION.UPDATED_SUCCESS')
                    .subscribe(function (res) { return _this.errorHandler.info(res); });
                _this.inProgress = false;
                _this.reload.emit(true);
                _this.close();
            }).catch(function (error) {
                _this.inProgress = false;
                _this.inlineAlert.showInlineError(error);
            });
        }
    };
    CreateEditRuleComponent.prototype.openCreateEditRule = function (ruleId) {
        var _this = this;
        this.initForm();
        this.selectedProjectList = [];
        this.filterCount = 0;
        this.filterListData = [];
        this.firstClick = 0;
        this.noSelectedProject = true;
        this.noSelectedEndpoint = true;
        this.isRuleNameExist = false;
        this.weeklySchedule = false;
        this.isScheduleOpt = false;
        this.isImmediate = false;
        this.policyId = -1;
        this.createEditRuleOpened = true;
        this.noProjectInfo = "";
        this.noEndpointInfo = "";
        if (this.targetList.length === 0) {
            this.noEndpointInfo = 'REPLICATION.NO_ENDPOINT_INFO';
        }
        if (this.projectList.length === 0 && !this.projectName) {
            this.noProjectInfo = 'REPLICATION.NO_PROJECT_INFO';
        }
        if (ruleId) {
            this.policyId = +ruleId;
            this.headerTitle = 'REPLICATION.EDIT_POLICY_TITLE';
            toPromise(this.repService.getReplicationRule(ruleId))
                .then(function (response) {
                _this.copyUpdateForm = Object.assign({}, response);
                // set filter value is [] if callback fiter value is null.
                _this.copyUpdateForm.filters = response.filters ? response.filters : [];
                _this.updateForm(response);
            }).catch(function (error) {
                _this.inlineAlert.showInlineError(error);
            });
        }
        else {
            this.headerTitle = 'REPLICATION.ADD_POLICY';
            if (this.projectId) {
                this.setProject([{ project_id: this.projectId, name: this.projectName }]);
                this.noSelectedProject = false;
            }
            this.copyUpdateForm = Object.assign({}, this.ruleForm.value);
        }
    };
    CreateEditRuleComponent.prototype.close = function () {
        this.createEditRuleOpened = false;
    };
    CreateEditRuleComponent.prototype.confirmCancel = function (confirmed) {
        this.inlineAlert.close();
        this.close();
    };
    CreateEditRuleComponent.prototype.onCancel = function () {
        if (this.hasFormChange()) {
            this.inlineAlert.showInlineConfirmation({ message: 'ALERT.FORM_CHANGE_CONFIRMATION' });
        }
        else {
            this.close();
        }
    };
    CreateEditRuleComponent.prototype.goRegistry = function () {
        this.goToRegistry.emit();
    };
    // UTC time
    CreateEditRuleComponent.prototype.getOfftime = function (daily_time) {
        var timeOffset = 0; // seconds
        if (daily_time && typeof daily_time === 'number') {
            timeOffset = +daily_time;
        }
        // Convert to current time
        var timezoneOffset = this._localTime.getTimezoneOffset();
        // Local time
        timeOffset = timeOffset - timezoneOffset * 60;
        if (timeOffset < 0) {
            timeOffset = timeOffset + ONE_DAY_SECONDS;
        }
        if (timeOffset >= ONE_DAY_SECONDS) {
            timeOffset -= ONE_DAY_SECONDS;
        }
        // To time string
        var hours = Math.floor(timeOffset / ONE_HOUR_SECONDS);
        var minutes = Math.floor((timeOffset - hours * ONE_HOUR_SECONDS) / 60);
        var timeStr = '' + hours;
        if (hours < 10) {
            timeStr = '0' + timeStr;
        }
        if (minutes < 10) {
            timeStr += ':0';
        }
        else {
            timeStr += ':';
        }
        timeStr += minutes;
        return timeStr;
    };
    CreateEditRuleComponent.prototype.setOfftime = function (v) {
        if (!v || v === '') {
            return;
        }
        var values = v.split(':');
        if (!values || values.length !== 2) {
            return;
        }
        var hours = +values[0];
        var minutes = +values[1];
        // Convert to UTC time
        var timezoneOffset = this._localTime.getTimezoneOffset();
        var utcTimes = hours * ONE_HOUR_SECONDS + minutes * 60;
        utcTimes += timezoneOffset * 60;
        if (utcTimes < 0) {
            utcTimes += ONE_DAY_SECONDS;
        }
        if (utcTimes >= ONE_DAY_SECONDS) {
            utcTimes -= ONE_DAY_SECONDS;
        }
        return utcTimes;
    };
    CreateEditRuleComponent.prototype.getChanges = function () {
        var changes = {};
        var ruleValue = this.ruleForm.value;
        if (!ruleValue || !this.copyUpdateForm) {
            return changes;
        }
        for (var prop in ruleValue) {
            var field = this.copyUpdateForm[prop];
            if (!compareValue(field, ruleValue[prop])) {
                if (ruleValue[prop][0] && ruleValue[prop][0].project_id && (ruleValue[prop][0].project_id === field[0].project_id)) {
                    break;
                }
                if (ruleValue[prop][0] && ruleValue[prop][0].id && (ruleValue[prop][0].id === field[0].id)) {
                    break;
                }
                changes[prop] = ruleValue[prop];
                // Number
                if (typeof field === "number") {
                    changes[prop] = +changes[prop];
                }
                // Trim string value
                if (typeof field === "string") {
                    changes[prop] = ('' + changes[prop]).trim();
                }
            }
        }
        return changes;
    };
    return CreateEditRuleComponent;
}());
export { CreateEditRuleComponent };
CreateEditRuleComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-create-edit-rule',
                template: CREATE_EDIT_RULE_TEMPLATE,
                styles: [CREATE_EDIT_RULE_STYLE]
            },] },
];
/** @nocollapse */
CreateEditRuleComponent.ctorParameters = function () { return [
    { type: FormBuilder, },
    { type: ReplicationService, },
    { type: EndpointService, },
    { type: ErrorHandler, },
    { type: ProjectService, },
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
]; };
CreateEditRuleComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'projectName': [{ type: Input },],
    'goToRegistry': [{ type: Output },],
    'reload': [{ type: Output },],
    'inlineAlert': [{ type: ViewChild, args: [InlineAlertComponent,] },],
};
//# sourceMappingURL=create-edit-rule.component.js.map