// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef } from '@angular/core';
import { EndpointService } from '../service/endpoint.service';
import { TranslateService } from '@ngx-translate/core';
import { ErrorHandler } from '../error-handler/index';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { CreateEditEndpointComponent } from '../create-edit-endpoint/create-edit-endpoint.component';
import { ENDPOINT_STYLE } from './endpoint.component.css';
import { ENDPOINT_TEMPLATE } from './endpoint.component.html';
import { toPromise, CustomComparator } from '../utils';
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
import { Observable } from "rxjs/Observable";
var EndpointComponent = (function () {
    function EndpointComponent(endpointService, errorHandler, translateService, ref) {
        this.endpointService = endpointService;
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.ref = ref;
        this.loading = false;
        this.creationTimeComparator = new CustomComparator('creation_time', 'date');
        this.selectedRow = [];
        this.batchDelectionInfos = [];
        this.forceRefreshView(1000);
    }
    Object.defineProperty(EndpointComponent.prototype, "initEndpoint", {
        get: function () {
            return {
                endpoint: "",
                name: "",
                username: "",
                password: "",
                insecure: false,
                type: 0
            };
        },
        enumerable: true,
        configurable: true
    });
    EndpointComponent.prototype.ngOnInit = function () {
        this.targetName = '';
        this.retrieve();
    };
    EndpointComponent.prototype.ngOnDestroy = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    EndpointComponent.prototype.selectedChange = function () {
        this.forceRefreshView(5000);
    };
    EndpointComponent.prototype.retrieve = function () {
        var _this = this;
        this.loading = true;
        this.selectedRow = [];
        toPromise(this.endpointService
            .getEndpoints(this.targetName))
            .then(function (targets) {
            _this.targets = targets || [];
            _this.forceRefreshView(1000);
            _this.loading = false;
        }).catch(function (error) {
            _this.errorHandler.error(error);
            _this.loading = false;
        });
    };
    EndpointComponent.prototype.doSearchTargets = function (targetName) {
        this.targetName = targetName;
        this.retrieve();
    };
    EndpointComponent.prototype.refreshTargets = function () {
        this.retrieve();
    };
    EndpointComponent.prototype.reload = function ($event) {
        this.targetName = '';
        this.retrieve();
    };
    EndpointComponent.prototype.openModal = function () {
        this.createEditEndpointComponent.openCreateEditTarget(true);
        this.target = this.initEndpoint;
    };
    EndpointComponent.prototype.editTargets = function (targets) {
        if (targets && targets.length === 1) {
            var target = targets[0];
            var editable = true;
            if (!target.id) {
                return;
            }
            var id = target.id;
            this.createEditEndpointComponent.openCreateEditTarget(editable, id);
        }
    };
    EndpointComponent.prototype.deleteTargets = function (targets) {
        var _this = this;
        if (targets && targets.length) {
            var targetNames_1 = [];
            this.batchDelectionInfos = [];
            targets.forEach(function (target) {
                targetNames_1.push(target.name);
                var initBatchMessage = new BatchInfo();
                initBatchMessage.name = target.name;
                _this.batchDelectionInfos.push(initBatchMessage);
            });
            var deletionMessage = new ConfirmationMessage('REPLICATION.DELETION_TITLE_TARGET', 'REPLICATION.DELETION_SUMMARY_TARGET', targetNames_1.join(', ') || '', targets, 6 /* TARGET */, 2 /* DELETE_CANCEL */);
            this.confirmationDialogComponent.open(deletionMessage);
        }
    };
    EndpointComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 6 /* TARGET */ &&
            message.state === 1 /* CONFIRMED */) {
            var targetLists = message.data;
            if (targetLists && targetLists.length) {
                var promiseLists_1 = [];
                targetLists.forEach(function (target) {
                    promiseLists_1.push(_this.delOperate(target.id, target.name));
                });
                Promise.all(promiseLists_1).then(function (item) {
                    _this.selectedRow = [];
                    _this.reload(true);
                    _this.forceRefreshView(2000);
                });
            }
        }
    };
    EndpointComponent.prototype.delOperate = function (id, name) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === name; });
        return toPromise(this.endpointService
            .deleteEndpoint(id))
            .then(function (response) {
            _this.translateService.get('BATCH.DELETED_SUCCESS')
                .subscribe(function (res) {
                findedList = BathInfoChanges(findedList, res);
            });
        }).catch(function (error) {
            if (error && error.status === 412) {
                Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('DESTINATION.FAILED_TO_DELETE_TARGET_IN_USED')).subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                });
            }
            else {
                _this.translateService.get('BATCH.DELETED_FAILURE').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            }
        });
    };
    //Forcely refresh the view
    EndpointComponent.prototype.forceRefreshView = function (duration) {
        var _this = this;
        //Reset timer
        if (this.timerHandler) {
            clearInterval(this.timerHandler);
        }
        this.timerHandler = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () {
            if (_this.timerHandler) {
                clearInterval(_this.timerHandler);
                _this.timerHandler = null;
            }
        }, duration);
    };
    return EndpointComponent;
}());
export { EndpointComponent };
EndpointComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-endpoint',
                template: ENDPOINT_TEMPLATE,
                styles: [ENDPOINT_STYLE],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
EndpointComponent.ctorParameters = function () { return [
    { type: EndpointService, },
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
]; };
EndpointComponent.propDecorators = {
    'createEditEndpointComponent': [{ type: ViewChild, args: [CreateEditEndpointComponent,] },],
    'confirmationDialogComponent': [{ type: ViewChild, args: ['confirmationDialog',] },],
};
//# sourceMappingURL=endpoint.component.js.map