/*
 * Copyright (c) 2017 VMware, Inc. All Rights Reserved.
 *
 * This product is licensed to you under the Apache License, Version 2.0 (the "License").
 * You may not use this product except in compliance with the License.
 *
 * This product may include a number of subcomponents with separate copyright notices
 * and license terms. Your use of these subcomponents is subject to the terms and
 * conditions of the subcomponent's license, as noted in the LICENSE file.
 */
import { Component, Input, Output, ContentChild, ViewChild, ViewChildren, TemplateRef, HostListener, ViewEncapsulation, EventEmitter } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { GRIDVIEW_TEMPLATE } from './grid-view.component.html';
import { GRIDVIEW_STYLE } from './grid-view.component.css';
/**
 * Grid view general component.
 */
var GridViewComponent = (function () {
    function GridViewComponent(translate) {
        this.translate = translate;
        this.expectScrollPercent = 70;
        this.loadNextPageEvent = new EventEmitter();
        this._items = [];
        this.cardStyles = [];
        this.itemsHolderStyle = {};
        this.loadedPages = 0;
        this.hidePartialRows = false;
        this.CurrentScrollPosition = {
            sH: 0,
            sT: 0,
            cH: 0
        };
        this.preScrollPosition = null;
    }
    Object.defineProperty(GridViewComponent.prototype, "items", {
        get: function () {
            return this._items;
        },
        set: function (value) {
            var _this = this;
            var newCardStyles = value.map(function (d, index) {
                if (index < _this.cardStyles.length) {
                    return _this.cardStyles[index];
                }
                return {
                    opacity: '0',
                    overflow: 'hidden'
                };
            });
            this.cardStyles = newCardStyles;
            this._items = value;
        },
        enumerable: true,
        configurable: true
    });
    GridViewComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.cards.changes.subscribe(function () {
            _this.throttleLayout();
        });
        this.throttleLayout();
    };
    GridViewComponent.prototype.onScroll = function (event) {
        this.preScrollPosition = this.CurrentScrollPosition;
        this.CurrentScrollPosition = {
            sH: event.target.scrollHeight,
            sT: event.target.scrollTop,
            cH: event.target.clientHeight
        };
        if (!this.loading
            && this.isScrollDown()
            && this.isScrollExpectPercent()
            && (this.currentPage * this.pageSize < this.totalCount)) {
            this.loadNextPageEvent.emit();
        }
    };
    GridViewComponent.prototype.isScrollDown = function () {
        return this.preScrollPosition.sT < this.CurrentScrollPosition.sT;
    };
    GridViewComponent.prototype.isScrollExpectPercent = function () {
        return ((this.CurrentScrollPosition.sT + this.CurrentScrollPosition.cH) / this.CurrentScrollPosition.sH) > (this.expectScrollPercent / 100);
    };
    GridViewComponent.prototype.onResize = function (event) {
        this.throttleLayout();
    };
    GridViewComponent.prototype.throttleLayout = function () {
        var _this = this;
        clearTimeout(this.layoutTimeout);
        this.layoutTimeout = setTimeout(function () {
            _this.layout.call(_this);
        }, 40);
    };
    Object.defineProperty(GridViewComponent.prototype, "isFirstPage", {
        get: function () {
            return this.currentPage <= 1;
        },
        enumerable: true,
        configurable: true
    });
    GridViewComponent.prototype.layout = function () {
        var el = this.itemsHolder.nativeElement;
        var width = el.offsetWidth;
        var items = el.querySelectorAll('.card-item');
        var items_count = items.length;
        if (items_count === 0) {
            el.height = 0;
            return;
        }
        var itemsHeight = [];
        for (var i = 0; i < items_count; i++) {
            itemsHeight[i] = items[i].offsetHeight;
        }
        var height = Math.max.apply(null, itemsHeight);
        var itemsStyle = window.getComputedStyle(items[0]);
        var minWidthStyle = itemsStyle.minWidth;
        var maxWidthStyle = itemsStyle.maxWidth;
        var minWidth = parseInt(minWidthStyle, 10);
        var maxWidth = parseInt(maxWidthStyle, 10);
        var marginHeight = parseInt(itemsStyle.marginTop, 10) + parseInt(itemsStyle.marginBottom, 10);
        var marginWidth = parseInt(itemsStyle.marginLeft, 10) + parseInt(itemsStyle.marginRight, 10);
        var columns = Math.floor(width / (minWidth + marginWidth));
        var columnsToUse = Math.max(Math.min(columns, items_count), 1);
        var rows = Math.floor(items_count / columnsToUse);
        var itemWidth = Math.min(Math.floor(width / columnsToUse) - marginWidth, maxWidth);
        var itemSpacing = columnsToUse === 1 || columns > items_count ? marginWidth :
            (width - marginWidth - columnsToUse * itemWidth) / (columnsToUse - 1);
        if (!this.withAdmiral) {
            // Fixed spacing and margin on standalone mode
            itemSpacing = marginWidth;
            itemWidth = minWidth;
        }
        var visible = items_count;
        if (this.hidePartialRows && this.totalItemsCount && items_count !== this.totalItemsCount) {
            visible = rows * columnsToUse;
        }
        var count = 0;
        for (var i = 0; i < visible; i++) {
            var item = items[i];
            var itemStyle = window.getComputedStyle(item);
            var left = (i % columnsToUse) * (itemWidth + itemSpacing);
            var top_1 = Math.floor(count / columnsToUse) * (height + marginHeight);
            // trick to show nice apear animation, where the item is already positioned,
            // but it will pop out
            var oldTransform = itemStyle.transform;
            if (!oldTransform || oldTransform === 'none') {
                this.cardStyles[i] = {
                    transform: 'translate(' + left + 'px,' + top_1 + 'px) scale(0)',
                    width: itemWidth + 'px',
                    transition: 'none',
                    overflow: 'hidden'
                };
                this.throttleLayout();
            }
            else {
                this.cardStyles[i] = {
                    transform: 'translate(' + left + 'px,' + top_1 + 'px) scale(1)',
                    width: itemWidth + 'px',
                    transition: null,
                    overflow: 'hidden'
                };
                this.throttleLayout();
            }
            if (!item.classList.contains('context-selected')) {
                var itemHeight = itemsHeight[i];
                if (itemStyle.display === 'none' && itemHeight !== 0) {
                    this.cardStyles[i].display = null;
                }
                if (itemHeight !== 0) {
                    count++;
                }
            }
        }
        for (var i = visible; i < items_count; i++) {
            this.cardStyles[i] = {
                display: 'none'
            };
        }
        this.itemsHolderStyle = {
            height: Math.ceil(count / columnsToUse) * (height + marginHeight) + 'px'
        };
    };
    GridViewComponent.prototype.onCardEnter = function (i) {
        this.cardStyles[i].overflow = 'visible';
    };
    GridViewComponent.prototype.onCardLeave = function (i) {
        this.cardStyles[i].overflow = 'hidden';
    };
    GridViewComponent.prototype.trackByFn = function (index, item) {
        return index;
    };
    return GridViewComponent;
}());
export { GridViewComponent };
GridViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-gridview',
                template: GRIDVIEW_TEMPLATE,
                styles: [GRIDVIEW_STYLE],
                encapsulation: ViewEncapsulation.None
            },] },
];
/** @nocollapse */
GridViewComponent.ctorParameters = function () { return [
    { type: TranslateService, },
]; };
GridViewComponent.propDecorators = {
    'loading': [{ type: Input },],
    'totalCount': [{ type: Input },],
    'currentPage': [{ type: Input },],
    'pageSize': [{ type: Input },],
    'expectScrollPercent': [{ type: Input },],
    'withAdmiral': [{ type: Input },],
    'items': [{ type: Input },],
    'loadNextPageEvent': [{ type: Output },],
    'cards': [{ type: ViewChildren, args: ['cardItem',] },],
    'itemsHolder': [{ type: ViewChild, args: ['itemsHolder',] },],
    'gridItemTmpl': [{ type: ContentChild, args: [TemplateRef,] },],
    'onScroll': [{ type: HostListener, args: ['scroll', ['$event'],] },],
    'onResize': [{ type: HostListener, args: ['window:resize',] },],
};
//# sourceMappingURL=grid-view.component.js.map