// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef, Input } from '@angular/core';
import { LABEL_TEMPLATE } from "./label.component.html";
import { LABEL_STYLE } from "./label.component.css";
import { LabelService } from "../service/label.service";
import { toPromise } from "../utils";
import { ErrorHandler } from "../error-handler/error-handler";
import { CreateEditLabelComponent } from "../create-edit-label/create-edit-label.component";
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
import { ConfirmationMessage } from "../confirmation-dialog/confirmation-message";
import { TranslateService } from "@ngx-translate/core";
var LabelComponent = (function () {
    function LabelComponent(labelService, errorHandler, translateService, ref) {
        this.labelService = labelService;
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.ref = ref;
        this.selectedRow = [];
        this.batchDelectionInfos = [];
        this.projectId = 0;
    }
    LabelComponent.prototype.ngOnInit = function () {
        this.retrieve(this.scope);
    };
    LabelComponent.prototype.retrieve = function (scope, name) {
        var _this = this;
        if (name === void 0) { name = ''; }
        this.loading = true;
        this.selectedRow = [];
        this.targetName = '';
        toPromise(this.labelService.getLabels(scope, this.projectId, name))
            .then(function (targets) {
            _this.targets = targets || [];
            _this.loading = false;
            _this.forceRefreshView(2000);
        }).catch(function (error) {
            _this.errorHandler.error(error);
            _this.loading = false;
        });
    };
    LabelComponent.prototype.openModal = function () {
        this.createEditLabel.openModal();
    };
    LabelComponent.prototype.reload = function () {
        this.retrieve(this.scope);
    };
    LabelComponent.prototype.doSearchTargets = function (targetName) {
        this.retrieve(this.scope, targetName);
    };
    LabelComponent.prototype.refreshTargets = function () {
        this.retrieve(this.scope);
    };
    LabelComponent.prototype.selectedChange = function () {
        // this.forceRefreshView(5000);
    };
    LabelComponent.prototype.editLabel = function (label) {
        this.createEditLabel.editModel(label[0].id, label);
    };
    LabelComponent.prototype.deleteLabels = function (targets) {
        var _this = this;
        if (targets && targets.length) {
            var targetNames_1 = [];
            this.batchDelectionInfos = [];
            targets.forEach(function (target) {
                targetNames_1.push(target.name);
                var initBatchMessage = new BatchInfo();
                initBatchMessage.name = target.name;
                _this.batchDelectionInfos.push(initBatchMessage);
            });
            var deletionMessage = new ConfirmationMessage('LABEL.DELETION_TITLE_TARGET', 'LABEL.DELETION_SUMMARY_TARGET', targetNames_1.join(', ') || '', targets, 6 /* TARGET */, 2 /* DELETE_CANCEL */);
            this.confirmationDialogComponent.open(deletionMessage);
        }
    };
    LabelComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 6 /* TARGET */ &&
            message.state === 1 /* CONFIRMED */) {
            var targetLists = message.data;
            if (targetLists && targetLists.length) {
                var promiseLists_1 = [];
                targetLists.forEach(function (target) {
                    promiseLists_1.push(_this.delOperate(target.id, target.name));
                });
                Promise.all(promiseLists_1).then(function (item) {
                    _this.selectedRow = [];
                    _this.retrieve(_this.scope);
                });
            }
        }
    };
    LabelComponent.prototype.delOperate = function (id, name) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === name; });
        return toPromise(this.labelService
            .deleteLabel(id))
            .then(function (response) {
            _this.translateService.get('BATCH.DELETED_SUCCESS')
                .subscribe(function (res) {
                findedList = BathInfoChanges(findedList, res);
            });
        }).catch(function (error) {
            _this.translateService.get('BATCH.DELETED_FAILURE').subscribe(function (res) {
                findedList = BathInfoChanges(findedList, res, false, true);
            });
        });
    };
    // Forcely refresh the view
    LabelComponent.prototype.forceRefreshView = function (duration) {
        var _this = this;
        // Reset timer
        if (this.timerHandler) {
            clearInterval(this.timerHandler);
        }
        this.timerHandler = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () {
            if (_this.timerHandler) {
                clearInterval(_this.timerHandler);
                _this.timerHandler = null;
            }
        }, duration);
    };
    return LabelComponent;
}());
export { LabelComponent };
LabelComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-label',
                template: LABEL_TEMPLATE,
                styles: [LABEL_STYLE],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
LabelComponent.ctorParameters = function () { return [
    { type: LabelService, },
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
]; };
LabelComponent.propDecorators = {
    'scope': [{ type: Input },],
    'projectId': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'createEditLabel': [{ type: ViewChild, args: [CreateEditLabelComponent,] },],
    'confirmationDialogComponent': [{ type: ViewChild, args: ['confirmationDialog',] },],
};
//# sourceMappingURL=label.component.js.map