// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, Input, Output, EventEmitter, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef } from '@angular/core';
import { ReplicationService } from '../service/replication.service';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { TranslateService } from '@ngx-translate/core';
import { ErrorHandler } from '../error-handler/error-handler';
import { toPromise, CustomComparator } from '../utils';
import { LIST_REPLICATION_RULE_TEMPLATE } from './list-replication-rule.component.html';
import { LIST_REPLICATION_RULE_CSS } from './list-replication-rule.component.css';
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
import { Observable } from "rxjs/Observable";
var ListReplicationRuleComponent = (function () {
    function ListReplicationRuleComponent(replicationService, translateService, errorHandler, ref) {
        this.replicationService = replicationService;
        this.translateService = translateService;
        this.errorHandler = errorHandler;
        this.ref = ref;
        this.nullTime = '0001-01-01T00:00:00Z';
        this.loading = false;
        this.reload = new EventEmitter();
        this.selectOne = new EventEmitter();
        this.editOne = new EventEmitter();
        this.toggleOne = new EventEmitter();
        this.hideJobs = new EventEmitter();
        this.redirect = new EventEmitter();
        this.openNewRule = new EventEmitter();
        this.replicateManual = new EventEmitter();
        this.projectScope = false;
        this.batchDelectionInfos = [];
        this.startTimeComparator = new CustomComparator('start_time', 'date');
        this.enabledComparator = new CustomComparator('enabled', 'number');
        setInterval(function () { return ref.markForCheck(); }, 500);
    }
    ListReplicationRuleComponent.prototype.trancatedDescription = function (desc) {
        if (desc.length > 35) {
            return desc.substr(0, 35);
        }
        else {
            return desc;
        }
    };
    ListReplicationRuleComponent.prototype.ngOnInit = function () {
        // Global scope
        if (!this.projectScope) {
            this.retrieveRules();
        }
    };
    ListReplicationRuleComponent.prototype.ngOnChanges = function (changes) {
        var proIdChange = changes["projectId"];
        if (proIdChange) {
            if (proIdChange.currentValue !== proIdChange.previousValue) {
                if (proIdChange.currentValue) {
                    this.projectId = proIdChange.currentValue;
                    this.projectScope = true; // Scope is project, not global list
                    // Initially load the replication rule data
                    this.retrieveRules();
                }
            }
        }
    };
    ListReplicationRuleComponent.prototype.retrieveRules = function (ruleName) {
        var _this = this;
        if (ruleName === void 0) { ruleName = ''; }
        this.loading = true;
        /*this.selectedRow = null;*/
        toPromise(this.replicationService
            .getReplicationRules(this.projectId, ruleName))
            .then(function (rules) {
            _this.rules = rules || [];
            // job list hidden
            _this.hideJobs.emit();
            _this.changedRules = _this.rules;
            _this.loading = false;
        }).catch(function (error) {
            _this.errorHandler.error(error);
            _this.loading = false;
        });
    };
    ListReplicationRuleComponent.prototype.replicateRule = function (rules) {
        this.replicateManual.emit(rules);
    };
    ListReplicationRuleComponent.prototype.deletionConfirm = function (message) {
        if (message &&
            message.source === 4 /* POLICY */ &&
            message.state === 1 /* CONFIRMED */) {
            this.deleteOpe(message.data);
        }
    };
    ListReplicationRuleComponent.prototype.selectRule = function (rule) {
        this.selectedId = rule.id || '';
        this.selectOne.emit(rule);
    };
    ListReplicationRuleComponent.prototype.redirectTo = function (rule) {
        this.redirect.emit(rule);
    };
    ListReplicationRuleComponent.prototype.openModal = function () {
        this.openNewRule.emit();
    };
    ListReplicationRuleComponent.prototype.editRule = function (rule) {
        this.editOne.emit(rule);
    };
    ListReplicationRuleComponent.prototype.jobList = function (id) {
        var _this = this;
        var ruleData;
        this.canDeleteRule = true;
        var count = 0;
        return toPromise(this.replicationService
            .getJobs(id))
            .then(function (response) {
            ruleData = response.data;
            if (ruleData.length) {
                ruleData.forEach(function (job) {
                    if ((job.status === 'pending') || (job.status === 'running') || (job.status === 'retrying')) {
                        count++;
                    }
                });
            }
            _this.canDeleteRule = count > 0 ? false : true;
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    ListReplicationRuleComponent.prototype.deleteRule = function (rule) {
        if (rule) {
            this.batchDelectionInfos = [];
            var initBatchMessage = new BatchInfo();
            initBatchMessage.name = rule.name;
            this.batchDelectionInfos.push(initBatchMessage);
            var deletionMessage = new ConfirmationMessage('REPLICATION.DELETION_TITLE', 'REPLICATION.DELETION_SUMMARY', rule.name, rule, 4 /* POLICY */, 2 /* DELETE_CANCEL */);
            this.deletionConfirmDialog.open(deletionMessage);
        }
    };
    ListReplicationRuleComponent.prototype.deleteOpe = function (rule) {
        var _this = this;
        if (rule) {
            var promiseLists_1 = [];
            Promise.all([this.jobList(rule.id)]).then(function (items) {
                if (!_this.canDeleteRule) {
                    var findedList_1 = _this.batchDelectionInfos.find(function (data) { return data.name === rule.name; });
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPLICATION.DELETION_SUMMARY_FAILURE')).subscribe(function (res) {
                        findedList_1 = BathInfoChanges(findedList_1, res[0], false, true, res[1]);
                    });
                }
                else {
                    promiseLists_1.push(_this.delOperate(+rule.id, rule.name));
                }
                Promise.all(promiseLists_1).then(function (item) {
                    _this.selectedRow = null;
                    _this.reload.emit(true);
                    var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 200);
                    setTimeout(function () { return clearInterval(hnd); }, 2000);
                });
            });
        }
    };
    ListReplicationRuleComponent.prototype.delOperate = function (ruleId, name) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === name; });
        return toPromise(this.replicationService
            .deleteReplicationRule(ruleId))
            .then(function () {
            _this.translateService.get('BATCH.DELETED_SUCCESS')
                .subscribe(function (res) { return findedList = BathInfoChanges(findedList, res); });
        })
            .catch(function (error) {
            if (error && error.status === 412) {
                Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPLICATION.FAILED_TO_DELETE_POLICY_ENABLED')).subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                });
            }
            else {
                _this.translateService.get('BATCH.DELETED_FAILURE').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            }
        });
    };
    return ListReplicationRuleComponent;
}());
export { ListReplicationRuleComponent };
ListReplicationRuleComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-list-replication-rule',
                template: LIST_REPLICATION_RULE_TEMPLATE,
                styles: [LIST_REPLICATION_RULE_CSS],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
ListReplicationRuleComponent.ctorParameters = function () { return [
    { type: ReplicationService, },
    { type: TranslateService, },
    { type: ErrorHandler, },
    { type: ChangeDetectorRef, },
]; };
ListReplicationRuleComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'isSystemAdmin': [{ type: Input },],
    'selectedId': [{ type: Input },],
    'withReplicationJob': [{ type: Input },],
    'loading': [{ type: Input },],
    'reload': [{ type: Output },],
    'selectOne': [{ type: Output },],
    'editOne': [{ type: Output },],
    'toggleOne': [{ type: Output },],
    'hideJobs': [{ type: Output },],
    'redirect': [{ type: Output },],
    'openNewRule': [{ type: Output },],
    'replicateManual': [{ type: Output },],
    'toggleConfirmDialog': [{ type: ViewChild, args: ['toggleConfirmDialog',] },],
    'deletionConfirmDialog': [{ type: ViewChild, args: ['deletionConfirmDialog',] },],
};
//# sourceMappingURL=list-replication-rule.component.js.map