// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, Input } from '@angular/core';
import { AccessLogService, RequestQueryParams } from '../service/index';
import { ErrorHandler } from '../error-handler/index';
import { toPromise, CustomComparator } from '../utils';
import { LOG_TEMPLATE, LOG_STYLES } from './recent-log.template';
import { DEFAULT_PAGE_SIZE, calculatePage, doFiltering, doSorting } from '../utils';
var RecentLogComponent = (function () {
    function RecentLogComponent(logService, errorHandler) {
        this.logService = logService;
        this.errorHandler = errorHandler;
        this.recentLogs = [];
        this.loading = true;
        this.withTitle = false;
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.currentPage = 1; //Double bound to pagination component
        this.currentPagePvt = 0; //Used to confirm whether page is changed
        this.opTimeComparator = new CustomComparator('op_time', 'date');
    }
    RecentLogComponent.prototype.ngOnInit = function () {
    };
    Object.defineProperty(RecentLogComponent.prototype, "totalCount", {
        get: function () {
            return this.logsCache && this.logsCache.metadata ? this.logsCache.metadata.xTotalCount : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecentLogComponent.prototype, "inProgress", {
        get: function () {
            return this.loading;
        },
        enumerable: true,
        configurable: true
    });
    RecentLogComponent.prototype.doFilter = function (terms) {
        this.currentTerm = terms.trim();
        //Trigger data loading and start from first page
        this.loading = true;
        this.currentPage = 1;
        if (this.currentPagePvt === 1) {
            //Force reloading
            var st = this.currentState;
            if (!st) {
                st = {
                    page: {}
                };
            }
            st.page.from = 0;
            st.page.to = this.pageSize - 1;
            st.page.size = this.pageSize;
            this.currentPagePvt = 0; //Reset pvt
            this.load(st);
        }
    };
    RecentLogComponent.prototype.refresh = function () {
        this.doFilter("");
    };
    RecentLogComponent.prototype.load = function (state) {
        var _this = this;
        //Keep it for future filter
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber !== this.currentPagePvt) {
            //load data
            var params = new RequestQueryParams();
            params.set("page", '' + pageNumber);
            params.set("page_size", '' + this.pageSize);
            if (this.currentTerm && this.currentTerm !== "") {
                params.set('repository', this.currentTerm);
            }
            this.loading = true;
            toPromise(this.logService.getRecentLogs(params))
                .then(function (response) {
                _this.logsCache = response; //Keep the data
                _this.recentLogs = _this.logsCache.data.filter(function (log) { return log.username != ""; }); //To display
                //Do customized filter
                _this.recentLogs = doFiltering(_this.recentLogs, state);
                //Do customized sorting
                _this.recentLogs = doSorting(_this.recentLogs, state);
                _this.currentPagePvt = pageNumber;
                _this.loading = false;
            })
                .catch(function (error) {
                _this.loading = false;
                _this.errorHandler.error(error);
            });
        }
        else {
            //Column sorting and filtering
            this.recentLogs = this.logsCache.data.filter(function (log) { return log.username != ""; }); //Reset data
            //Do customized filter
            this.recentLogs = doFiltering(this.recentLogs, state);
            //Do customized sorting
            this.recentLogs = doSorting(this.recentLogs, state);
        }
    };
    RecentLogComponent.prototype.isMatched = function (terms, log) {
        var reg = new RegExp('.*' + terms + '.*', 'i');
        return reg.test(log.username) ||
            reg.test(log.repo_name) ||
            reg.test(log.operation) ||
            reg.test(log.repo_tag);
    };
    return RecentLogComponent;
}());
export { RecentLogComponent };
RecentLogComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-log',
                styles: [LOG_STYLES],
                template: LOG_TEMPLATE
            },] },
];
/** @nocollapse */
RecentLogComponent.ctorParameters = function () { return [
    { type: AccessLogService, },
    { type: ErrorHandler, },
]; };
RecentLogComponent.propDecorators = {
    'withTitle': [{ type: Input },],
};
//# sourceMappingURL=recent-log.component.js.map