// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { ListReplicationRuleComponent } from '../list-replication-rule/list-replication-rule.component';
import { CreateEditRuleComponent } from '../create-edit-rule/create-edit-rule.component';
import { ErrorHandler } from '../error-handler/error-handler';
import { ReplicationService } from '../service/replication.service';
import { RequestQueryParams } from '../service/RequestQueryParams';
import { toPromise, CustomComparator, DEFAULT_PAGE_SIZE, doFiltering, doSorting, calculatePage } from '../utils';
import { REPLICATION_TEMPLATE } from './replication.component.html';
import { REPLICATION_STYLE } from './replication.component.css';
import { Observable } from "rxjs/Observable";
import { ConfirmationMessage } from "../confirmation-dialog/confirmation-message";
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
var ruleStatus = [
    { 'key': 'all', 'description': 'REPLICATION.ALL_STATUS' },
    { 'key': '1', 'description': 'REPLICATION.ENABLED' },
    { 'key': '0', 'description': 'REPLICATION.DISABLED' }
];
var jobStatus = [
    { 'key': 'all', 'description': 'REPLICATION.ALL' },
    { 'key': 'pending', 'description': 'REPLICATION.PENDING' },
    { 'key': 'running', 'description': 'REPLICATION.RUNNING' },
    { 'key': 'error', 'description': 'REPLICATION.ERROR' },
    { 'key': 'retrying', 'description': 'REPLICATION.RETRYING' },
    { 'key': 'stopped', 'description': 'REPLICATION.STOPPED' },
    { 'key': 'finished', 'description': 'REPLICATION.FINISHED' },
    { 'key': 'canceled', 'description': 'REPLICATION.CANCELED' }
];
var optionalSearch = { 0: 'REPLICATION.ADVANCED', 1: 'REPLICATION.SIMPLE' };
var SearchOption = (function () {
    function SearchOption() {
        this.ruleName = '';
        this.repoName = '';
        this.status = '';
        this.startTime = '';
        this.startTimestamp = '';
        this.endTime = '';
        this.endTimestamp = '';
        this.page = 1;
        this.pageSize = DEFAULT_PAGE_SIZE;
    }
    return SearchOption;
}());
export { SearchOption };
var ReplicationComponent = (function () {
    function ReplicationComponent(errorHandler, replicationService, translateService) {
        this.errorHandler = errorHandler;
        this.replicationService = replicationService;
        this.translateService = translateService;
        this.redirect = new EventEmitter();
        this.openCreateRule = new EventEmitter();
        this.openEdit = new EventEmitter();
        this.goToRegistry = new EventEmitter();
        this.search = new SearchOption();
        this.ruleStatus = ruleStatus;
        this.jobStatus = jobStatus;
        this.hiddenJobList = true;
        this.batchDelectionInfos = [];
        this.toggleJobSearchOption = optionalSearch;
        this.creationTimeComparator = new CustomComparator('creation_time', 'date');
        this.updateTimeComparator = new CustomComparator('update_time', 'date');
        //Server driven pagination
        this.currentPage = 1;
        this.totalCount = 0;
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.jobsLoading = false;
    }
    Object.defineProperty(ReplicationComponent.prototype, "showPaginationIndex", {
        get: function () {
            return this.totalCount > 0;
        },
        enumerable: true,
        configurable: true
    });
    ReplicationComponent.prototype.ngOnInit = function () {
        this.currentRuleStatus = this.ruleStatus[0];
        this.currentJobStatus = this.jobStatus[0];
        this.currentJobSearchOption = 0;
    };
    ReplicationComponent.prototype.ngOnDestroy = function () {
        if (this.timerDelay) {
            this.timerDelay.unsubscribe();
        }
    };
    // open replication rule
    ReplicationComponent.prototype.openModal = function () {
        this.createEditPolicyComponent.openCreateEditRule();
    };
    // edit replication rule
    ReplicationComponent.prototype.openEditRule = function (rule) {
        if (rule) {
            this.createEditPolicyComponent.openCreateEditRule(rule.id);
        }
    };
    ReplicationComponent.prototype.goRegistry = function () {
        this.goToRegistry.emit();
    };
    //Server driven data loading
    ReplicationComponent.prototype.clrLoadJobs = function (state) {
        var _this = this;
        if (!state || !state.page || !this.search.ruleId) {
            return;
        }
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        var params = new RequestQueryParams();
        //Pagination
        params.set("page", '' + pageNumber);
        params.set("page_size", '' + this.pageSize);
        //Search by status
        if (this.search.status.trim()) {
            params.set('status', this.search.status);
        }
        //Search by repository
        if (this.search.repoName.trim()) {
            params.set('repository', this.search.repoName);
        }
        //Search by timestamps
        if (this.search.startTimestamp.trim()) {
            params.set('start_time', this.search.startTimestamp);
        }
        if (this.search.endTimestamp.trim()) {
            params.set('end_time', this.search.endTimestamp);
        }
        this.jobsLoading = true;
        //Do filtering and sorting
        this.jobs = doFiltering(this.jobs, state);
        this.jobs = doSorting(this.jobs, state);
        this.jobsLoading = false;
        toPromise(this.replicationService
            .getJobs(this.search.ruleId, params))
            .then(function (response) {
            _this.totalCount = response.metadata.xTotalCount;
            _this.jobs = response.data;
            if (!_this.timerDelay) {
                _this.timerDelay = Observable.timer(10000, 10000).subscribe(function () {
                    var count = 0;
                    _this.jobs.forEach(function (job) {
                        if ((job.status === 'pending') || (job.status === 'running') || (job.status === 'retrying')) {
                            count++;
                        }
                    });
                    if (count > 0) {
                        _this.clrLoadJobs(_this.currentState);
                    }
                    else {
                        _this.timerDelay.unsubscribe();
                        _this.timerDelay = null;
                    }
                });
            }
            //Do filtering and sorting
            _this.jobs = doFiltering(_this.jobs, state);
            _this.jobs = doSorting(_this.jobs, state);
            _this.jobsLoading = false;
        }).catch(function (error) {
            _this.jobsLoading = false;
            _this.errorHandler.error(error);
        });
    };
    ReplicationComponent.prototype.loadFirstPage = function () {
        var st = this.currentState;
        if (!st) {
            st = {
                page: {}
            };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        this.clrLoadJobs(st);
    };
    ReplicationComponent.prototype.selectOneRule = function (rule) {
        if (rule && rule.id) {
            this.hiddenJobList = false;
            this.search.ruleId = rule.id || '';
            this.search.repoName = '';
            this.search.status = '';
            this.currentJobSearchOption = 0;
            this.currentJobStatus = { 'key': 'all', 'description': 'REPLICATION.ALL' };
            this.loadFirstPage();
        }
    };
    ReplicationComponent.prototype.replicateManualRule = function (rule) {
        if (rule) {
            this.batchDelectionInfos = [];
            var initBatchMessage = new BatchInfo();
            initBatchMessage.name = rule.name;
            this.batchDelectionInfos.push(initBatchMessage);
            var replicationMessage = new ConfirmationMessage('REPLICATION.REPLICATION_TITLE', 'REPLICATION.REPLICATION_SUMMARY', rule.name, rule, 6 /* TARGET */, 4 /* REPLICATE_CANCEL */);
            this.replicationConfirmDialog.open(replicationMessage);
        }
    };
    ReplicationComponent.prototype.confirmReplication = function (message) {
        var _this = this;
        if (message &&
            message.source === 6 /* TARGET */ &&
            message.state === 1 /* CONFIRMED */) {
            var rule_1 = message.data;
            if (rule_1) {
                Promise.all([this.replicationOperate(+rule_1.id, rule_1.name)]).then(function (item) {
                    _this.selectOneRule(rule_1);
                });
            }
        }
    };
    ReplicationComponent.prototype.replicationOperate = function (ruleId, name) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === name; });
        return toPromise(this.replicationService.replicateRule(ruleId))
            .then(function (response) {
            _this.translateService.get('BATCH.REPLICATE_SUCCESS')
                .subscribe(function (res) { return findedList = BathInfoChanges(findedList, res); });
        })
            .catch(function (error) {
            if (error && error.status === 412) {
                Observable.forkJoin(_this.translateService.get('BATCH.REPLICATE_FAILURE'), _this.translateService.get('REPLICATION.REPLICATE_SUMMARY_FAILURE'))
                    .subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                });
            }
            else {
                _this.translateService.get('BATCH.REPLICATE_FAILURE').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            }
        });
    };
    ReplicationComponent.prototype.customRedirect = function (rule) {
        this.redirect.emit(rule);
    };
    ReplicationComponent.prototype.doSearchRules = function (ruleName) {
        this.search.ruleName = ruleName;
        this.listReplicationRule.retrieveRules(ruleName);
    };
    ReplicationComponent.prototype.doFilterJobStatus = function ($event) {
        if ($event && $event.target && $event.target["value"]) {
            var status_1 = $event.target["value"];
            this.currentJobStatus = this.jobStatus.find(function (r) { return r.key === status_1; });
            if (this.currentJobStatus.key === 'all') {
                status_1 = '';
            }
            this.search.status = status_1;
            this.doSearchJobs(this.search.repoName);
        }
    };
    ReplicationComponent.prototype.doSearchJobs = function (repoName) {
        this.search.repoName = repoName;
        this.loadFirstPage();
    };
    ReplicationComponent.prototype.hideJobs = function () {
        this.search.ruleId = 0;
        this.jobs = [];
        this.hiddenJobList = true;
    };
    ReplicationComponent.prototype.stopJobs = function () {
        var _this = this;
        if (this.jobs && this.jobs.length) {
            this.isStopOnGoing = true;
            toPromise(this.replicationService.stopJobs(this.jobs[0].policy_id))
                .then(function (res) {
                _this.refreshJobs();
                _this.isStopOnGoing = false;
            })
                .catch(function (error) { return _this.errorHandler.error(error); });
        }
    };
    ReplicationComponent.prototype.reloadRules = function (isReady) {
        if (isReady) {
            this.search.ruleName = '';
            this.listReplicationRule.retrieveRules(this.search.ruleName);
        }
    };
    ReplicationComponent.prototype.refreshRules = function () {
        this.listReplicationRule.retrieveRules();
    };
    ReplicationComponent.prototype.refreshJobs = function () {
        this.currentJobStatus = this.jobStatus[0];
        this.search.startTime = ' ';
        this.search.endTime = ' ';
        this.search.repoName = "";
        this.search.startTimestamp = "";
        this.search.endTimestamp = "";
        this.search.status = "";
        this.currentPage = 1;
        var st = {
            page: {
                from: 0,
                to: this.pageSize - 1,
                size: this.pageSize
            }
        };
        this.clrLoadJobs(st);
    };
    ReplicationComponent.prototype.toggleSearchJobOptionalName = function (option) {
        (option === 1) ? this.currentJobSearchOption = 0 : this.currentJobSearchOption = 1;
    };
    ReplicationComponent.prototype.doJobSearchByStartTime = function (fromTimestamp) {
        this.search.startTimestamp = fromTimestamp;
        this.loadFirstPage();
    };
    ReplicationComponent.prototype.doJobSearchByEndTime = function (toTimestamp) {
        this.search.endTimestamp = toTimestamp;
        this.loadFirstPage();
    };
    ReplicationComponent.prototype.viewLog = function (jobId) {
        if (this.replicationLogViewer) {
            this.replicationLogViewer.open(jobId);
        }
    };
    return ReplicationComponent;
}());
export { ReplicationComponent };
ReplicationComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-replication',
                template: REPLICATION_TEMPLATE,
                styles: [REPLICATION_STYLE]
            },] },
];
/** @nocollapse */
ReplicationComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: ReplicationService, },
    { type: TranslateService, },
]; };
ReplicationComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'projectName': [{ type: Input },],
    'isSystemAdmin': [{ type: Input },],
    'withReplicationJob': [{ type: Input },],
    'redirect': [{ type: Output },],
    'openCreateRule': [{ type: Output },],
    'openEdit': [{ type: Output },],
    'goToRegistry': [{ type: Output },],
    'listReplicationRule': [{ type: ViewChild, args: [ListReplicationRuleComponent,] },],
    'createEditPolicyComponent': [{ type: ViewChild, args: [CreateEditRuleComponent,] },],
    'replicationLogViewer': [{ type: ViewChild, args: ["replicationLogViewer",] },],
    'replicationConfirmDialog': [{ type: ViewChild, args: ['replicationConfirmDialog',] },],
};
//# sourceMappingURL=replication.component.js.map