import { Component, Input, Output, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef, EventEmitter } from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from "rxjs/Observable";
import { TranslateService } from '@ngx-translate/core';
import { REPOSITORY_GRIDVIEW_TEMPLATE } from './repository-gridview.component.html';
import { REPOSITORY_GRIDVIEW_STYLE } from './repository-gridview.component.css';
import { SystemInfoService, RepositoryService, RequestQueryParams, TagService } from '../service/index';
import { ErrorHandler } from '../error-handler/error-handler';
import { toPromise, CustomComparator, DEFAULT_PAGE_SIZE, calculatePage, doFiltering, doSorting, clone } from '../utils';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
var RepositoryGridviewComponent = (function () {
    function RepositoryGridviewComponent(errorHandler, translateService, repositoryService, systemInfoService, tagService, ref, router) {
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.repositoryService = repositoryService;
        this.systemInfoService = systemInfoService;
        this.tagService = tagService;
        this.ref = ref;
        this.router = router;
        this.signedCon = {};
        this.projectName = 'unknown';
        this.mode = 'admiral';
        this.repoClickEvent = new EventEmitter();
        this.repoProvisionEvent = new EventEmitter();
        this.addInfoEvent = new EventEmitter();
        this.repositories = [];
        this.repositoriesCopy = [];
        this.selectedRow = [];
        this.loading = true;
        this.cardHover = false;
        this.listHover = false;
        this.batchDelectionInfos = [];
        this.pullCountComparator = new CustomComparator('pull_count', 'number');
        this.tagsCountComparator = new CustomComparator('tags_count', 'number');
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.currentPage = 1;
        this.totalCount = 0;
    }
    Object.defineProperty(RepositoryGridviewComponent.prototype, "registryUrl", {
        get: function () {
            return this.systemInfo ? this.systemInfo.registry_url : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryGridviewComponent.prototype, "withClair", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_clair : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryGridviewComponent.prototype, "isClairDBReady", {
        get: function () {
            return this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.overall_last_update > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryGridviewComponent.prototype, "withAdmiral", {
        get: function () {
            return this.mode === 'admiral';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryGridviewComponent.prototype, "showDBStatusWarning", {
        get: function () {
            return this.withClair && !this.isClairDBReady;
        },
        enumerable: true,
        configurable: true
    });
    RepositoryGridviewComponent.prototype.ngOnChanges = function (changes) {
        if (changes['projectId'] && changes['projectId'].currentValue) {
            this.refresh();
        }
    };
    RepositoryGridviewComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Get system info for tag views
        toPromise(this.systemInfoService.getSystemInfo())
            .then(function (systemInfo) { return _this.systemInfo = systemInfo; })
            .catch(function (error) { return _this.errorHandler.error(error); });
        if (this.mode === 'admiral') {
            this.isCardView = true;
        }
        else {
            this.isCardView = false;
        }
        this.lastFilteredRepoName = '';
    };
    RepositoryGridviewComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 7 /* REPOSITORY */ &&
            message.state === 1 /* CONFIRMED */) {
            var promiseLists_1 = [];
            var repoNames = message.data.split(',');
            repoNames.forEach(function (repoName) {
                promiseLists_1.push(_this.delOperate(repoName));
            });
            Promise.all(promiseLists_1).then(function (item) {
                _this.selectedRow = [];
                _this.refresh();
                var st = _this.getStateAfterDeletion();
                if (!st) {
                    _this.refresh();
                }
                else {
                    _this.clrLoad(st);
                }
            });
        }
    };
    RepositoryGridviewComponent.prototype.delOperate = function (repoName) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === repoName; });
        if (this.signedCon[repoName].length !== 0) {
            Observable.forkJoin(this.translateService.get('BATCH.DELETED_FAILURE'), this.translateService.get('REPOSITORY.DELETION_TITLE_REPO_SIGNED')).subscribe(function (res) {
                findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
            });
        }
        else {
            return toPromise(this.repositoryService
                .deleteRepository(repoName))
                .then(function (response) {
                _this.translateService.get('BATCH.DELETED_SUCCESS').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res);
                });
            }).catch(function (error) {
                if (error.status === "412") {
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPOSITORY.TAGS_SIGNED')).subscribe(function (res) {
                        findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                    });
                    return;
                }
                if (error.status === 503) {
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPOSITORY.TAGS_NO_DELETE')).subscribe(function (res) {
                        findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                    });
                    return;
                }
                _this.translateService.get('BATCH.DELETED_FAILURE').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            });
        }
    };
    RepositoryGridviewComponent.prototype.doSearchRepoNames = function (repoName) {
        this.lastFilteredRepoName = repoName;
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        this.clrLoad(st);
    };
    RepositoryGridviewComponent.prototype.saveSignatures = function (event) {
        Object.assign(this.signedCon, event);
    };
    RepositoryGridviewComponent.prototype.deleteRepos = function (repoLists) {
        var _this = this;
        if (repoLists && repoLists.length) {
            var repoNames_1 = [];
            this.batchDelectionInfos = [];
            var repArr_1 = [];
            repoLists.forEach(function (repo) {
                repoNames_1.push(repo.name);
                var initBatchMessage = new BatchInfo();
                initBatchMessage.name = repo.name;
                _this.batchDelectionInfos.push(initBatchMessage);
                if (!_this.signedCon[repo.name]) {
                    repArr_1.push(_this.getTagInfo(repo.name));
                }
            });
            Promise.all(repArr_1).then(function () {
                _this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO', '', repoNames_1.join(','), 'REPOSITORY.DELETION_SUMMARY_REPO', 2 /* DELETE_CANCEL */);
            });
        }
    };
    RepositoryGridviewComponent.prototype.getTagInfo = function (repoName) {
        var _this = this;
        this.signedCon[repoName] = [];
        return toPromise(this.tagService
            .getTags(repoName))
            .then(function (items) {
            items.forEach(function (t) {
                if (t.signature !== null) {
                    _this.signedCon[repoName].push(t.name);
                }
            });
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryGridviewComponent.prototype.signedDataSet = function (repoName) {
        var signature = '';
        if (this.signedCon[repoName].length === 0) {
            this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO', 2 /* DELETE_CANCEL */);
            return;
        }
        signature = this.signedCon[repoName].join(',');
        this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO_SIGNED', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO_SIGNED', 3 /* CLOSE */);
    };
    RepositoryGridviewComponent.prototype.confirmationDialogSet = function (summaryTitle, signature, repoName, summaryKey, button) {
        var _this = this;
        this.translateService.get(summaryKey, {
            repoName: repoName,
            signedImages: signature,
        })
            .subscribe(function (res) {
            summaryKey = res;
            var message = new ConfirmationMessage(summaryTitle, summaryKey, repoName, repoName, 7 /* REPOSITORY */, button);
            _this.confirmationDialog.open(message);
            var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
            setTimeout(function () { return clearInterval(hnd); }, 5000);
        });
    };
    RepositoryGridviewComponent.prototype.containsLatestTag = function (repo) {
        return toPromise(this.tagService.getTags(repo.name))
            .then(function (items) {
            if (items.some(function (t) { return t.name === 'latest'; })) {
                return true;
            }
            else {
                return false;
            }
            ;
        })
            .catch(function (error) { return Promise.reject(false); });
    };
    RepositoryGridviewComponent.prototype.provisionItemEvent = function (evt, repo) {
        var _this = this;
        evt.stopPropagation();
        var repoCopy = clone(repo);
        repoCopy.name = this.registryUrl + ":443/" + repoCopy.name;
        this.containsLatestTag(repo)
            .then(function (containsLatest) {
            if (containsLatest) {
                _this.repoProvisionEvent.emit(repoCopy);
            }
            else {
                _this.addInfoEvent.emit(repoCopy);
            }
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryGridviewComponent.prototype.itemAddInfoEvent = function (evt, repo) {
        evt.stopPropagation();
        var repoCopy = clone(repo);
        repoCopy.name = this.registryUrl + ':443/' + repoCopy.name;
        this.addInfoEvent.emit(repoCopy);
    };
    RepositoryGridviewComponent.prototype.deleteItemEvent = function (evt, item) {
        evt.stopPropagation();
        this.deleteRepos([item]);
    };
    RepositoryGridviewComponent.prototype.selectedChange = function () {
        var _this = this;
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 2000);
    };
    RepositoryGridviewComponent.prototype.refresh = function () {
        this.doSearchRepoNames('');
    };
    RepositoryGridviewComponent.prototype.loadNextPage = function () {
        var _this = this;
        if (this.currentPage * this.pageSize >= this.totalCount) {
            return;
        }
        this.currentPage = this.currentPage + 1;
        // Pagination
        var params = new RequestQueryParams();
        params.set("page", '' + this.currentPage);
        params.set("page_size", '' + this.pageSize);
        this.loading = true;
        toPromise(this.repositoryService.getRepositories(this.projectId, this.lastFilteredRepoName, params))
            .then(function (repo) {
            _this.totalCount = repo.metadata.xTotalCount;
            _this.repositoriesCopy = repo.data;
            _this.signedCon = {};
            // Do filtering and sorting
            _this.repositoriesCopy = doFiltering(_this.repositoriesCopy, _this.currentState);
            _this.repositoriesCopy = doSorting(_this.repositoriesCopy, _this.currentState);
            _this.repositories = _this.repositories.concat(_this.repositoriesCopy);
            _this.loading = false;
        })
            .catch(function (error) {
            _this.loading = false;
            _this.errorHandler.error(error);
        });
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 500);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    RepositoryGridviewComponent.prototype.clrLoad = function (state) {
        var _this = this;
        this.selectedRow = [];
        // Keep it for future filtering and sorting
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        // Pagination
        var params = new RequestQueryParams();
        params.set("page", '' + pageNumber);
        params.set("page_size", '' + this.pageSize);
        this.loading = true;
        toPromise(this.repositoryService.getRepositories(this.projectId, this.lastFilteredRepoName, params))
            .then(function (repo) {
            _this.totalCount = repo.metadata.xTotalCount;
            _this.repositories = repo.data;
            _this.signedCon = {};
            // Do filtering and sorting
            _this.repositories = doFiltering(_this.repositories, state);
            _this.repositories = doSorting(_this.repositories, state);
            _this.loading = false;
        })
            .catch(function (error) {
            _this.loading = false;
            _this.errorHandler.error(error);
        });
        // Force refresh view
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    RepositoryGridviewComponent.prototype.getStateAfterDeletion = function () {
        var total = this.totalCount - 1;
        if (total <= 0) {
            return null;
        }
        var totalPages = Math.ceil(total / this.pageSize);
        var targetPageNumber = this.currentPage;
        if (this.currentPage > totalPages) {
            targetPageNumber = totalPages; // Should == currentPage -1
        }
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = (targetPageNumber - 1) * this.pageSize;
        st.page.to = targetPageNumber * this.pageSize - 1;
        return st;
    };
    RepositoryGridviewComponent.prototype.watchRepoClickEvt = function (repo) {
        this.repoClickEvent.emit(repo);
    };
    RepositoryGridviewComponent.prototype.getImgLink = function (repo) {
        return '/container-image-icons?container-image=' + repo.name;
    };
    RepositoryGridviewComponent.prototype.getRepoDescrition = function (repo) {
        if (repo && repo.description) {
            return repo.description;
        }
        return "No description for this repo. You can add it to this repository.";
    };
    RepositoryGridviewComponent.prototype.showCard = function (cardView) {
        if (this.isCardView === cardView) {
            return;
        }
        this.isCardView = cardView;
        this.refresh();
    };
    RepositoryGridviewComponent.prototype.mouseEnter = function (itemName) {
        if (itemName === 'card') {
            this.cardHover = true;
        }
        else {
            this.listHover = true;
        }
    };
    RepositoryGridviewComponent.prototype.mouseLeave = function (itemName) {
        if (itemName === 'card') {
            this.cardHover = false;
        }
        else {
            this.listHover = false;
        }
    };
    RepositoryGridviewComponent.prototype.isHovering = function (itemName) {
        if (itemName === 'card') {
            return this.cardHover;
        }
        else {
            return this.listHover;
        }
    };
    return RepositoryGridviewComponent;
}());
export { RepositoryGridviewComponent };
RepositoryGridviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-repository-gridview',
                template: REPOSITORY_GRIDVIEW_TEMPLATE,
                styles: [REPOSITORY_GRIDVIEW_STYLE],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
RepositoryGridviewComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: RepositoryService, },
    { type: SystemInfoService, },
    { type: TagService, },
    { type: ChangeDetectorRef, },
    { type: Router, },
]; };
RepositoryGridviewComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'projectName': [{ type: Input },],
    'urlPrefix': [{ type: Input },],
    'hasSignedIn': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'mode': [{ type: Input },],
    'repoClickEvent': [{ type: Output },],
    'repoProvisionEvent': [{ type: Output },],
    'addInfoEvent': [{ type: Output },],
    'confirmationDialog': [{ type: ViewChild, args: ['confirmationDialog',] },],
    'gridView': [{ type: ViewChild, args: ['gridView',] },],
};
//# sourceMappingURL=repository-gridview.component.js.map