import { Component, Input, Output, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef, EventEmitter } from '@angular/core';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { REPOSITORY_LISTVIEW_TEMPLATE } from './repository-listview.component.html';
import { REPOSITORY_LISTVIEW_STYLE } from './repository-listview.component.css';
import { SystemInfoService, RepositoryService, RequestQueryParams, TagService } from '../service/index';
import { ErrorHandler } from '../error-handler/error-handler';
import { toPromise, CustomComparator } from '../utils';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { DEFAULT_PAGE_SIZE, calculatePage, doFiltering, doSorting } from '../utils';
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
import { Observable } from "rxjs/Observable";
var RepositoryListviewComponent = (function () {
    function RepositoryListviewComponent(errorHandler, translateService, repositoryService, systemInfoService, tagService, ref, router) {
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.repositoryService = repositoryService;
        this.systemInfoService = systemInfoService;
        this.tagService = tagService;
        this.ref = ref;
        this.router = router;
        this.signedCon = {};
        this.projectName = 'unknown';
        this.repoClickEvent = new EventEmitter();
        this.selectedRow = [];
        this.loading = true;
        this.batchDelectionInfos = [];
        this.pullCountComparator = new CustomComparator('pull_count', 'number');
        this.tagsCountComparator = new CustomComparator('tags_count', 'number');
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.currentPage = 1;
        this.totalCount = 0;
    }
    Object.defineProperty(RepositoryListviewComponent.prototype, "registryUrl", {
        get: function () {
            return this.systemInfo ? this.systemInfo.registry_url : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryListviewComponent.prototype, "withNotary", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_notary : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryListviewComponent.prototype, "withClair", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_clair : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryListviewComponent.prototype, "isClairDBReady", {
        get: function () {
            return this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.overall_last_update > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryListviewComponent.prototype, "showDBStatusWarning", {
        get: function () {
            return this.withClair && !this.isClairDBReady;
        },
        enumerable: true,
        configurable: true
    });
    RepositoryListviewComponent.prototype.ngOnChanges = function (changes) {
        if (changes['projectId'] && changes['projectId'].currentValue) {
            this.refresh();
        }
    };
    RepositoryListviewComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Get system info for tag views
        toPromise(this.systemInfoService.getSystemInfo())
            .then(function (systemInfo) { return _this.systemInfo = systemInfo; })
            .catch(function (error) { return _this.errorHandler.error(error); });
        this.lastFilteredRepoName = '';
    };
    RepositoryListviewComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 7 /* REPOSITORY */ &&
            message.state === 1 /* CONFIRMED */) {
            var promiseLists_1 = [];
            var repoNames = message.data.split(',');
            repoNames.forEach(function (repoName) {
                promiseLists_1.push(_this.delOperate(repoName));
            });
            Promise.all(promiseLists_1).then(function (item) {
                _this.selectedRow = [];
                _this.refresh();
                var st = _this.getStateAfterDeletion();
                if (!st) {
                    _this.refresh();
                }
                else {
                    _this.clrLoad(st);
                }
            });
        }
    };
    RepositoryListviewComponent.prototype.delOperate = function (repoName) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === repoName; });
        if (this.signedCon[repoName].length !== 0) {
            Observable.forkJoin(this.translateService.get('BATCH.DELETED_FAILURE'), this.translateService.get('REPOSITORY.DELETION_TITLE_REPO_SIGNED')).subscribe(function (res) {
                findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
            });
        }
        else {
            return toPromise(this.repositoryService
                .deleteRepository(repoName))
                .then(function (response) {
                _this.translateService.get('BATCH.DELETED_SUCCESS').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res);
                });
            }).catch(function (error) {
                if (error.status === "412") {
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPOSITORY.TAGS_SIGNED')).subscribe(function (res) {
                        findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                    });
                    return;
                }
                if (error.status === 503) {
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPOSITORY.TAGS_NO_DELETE')).subscribe(function (res) {
                        findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                    });
                    return;
                }
                _this.translateService.get('BATCH.DELETED_FAILURE').subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            });
        }
    };
    RepositoryListviewComponent.prototype.doSearchRepoNames = function (repoName) {
        this.lastFilteredRepoName = repoName;
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        this.clrLoad(st);
    };
    RepositoryListviewComponent.prototype.saveSignatures = function (event) {
        Object.assign(this.signedCon, event);
    };
    RepositoryListviewComponent.prototype.deleteRepos = function (repoLists) {
        var _this = this;
        if (repoLists && repoLists.length) {
            var repoNames_1 = [];
            this.batchDelectionInfos = [];
            var repArr_1 = [];
            repoLists.forEach(function (repo) {
                repoNames_1.push(repo.name);
                var initBatchMessage = new BatchInfo();
                initBatchMessage.name = repo.name;
                _this.batchDelectionInfos.push(initBatchMessage);
                if (!_this.signedCon[repo.name]) {
                    repArr_1.push(_this.getTagInfo(repo.name));
                }
            });
            Promise.all(repArr_1).then(function () {
                _this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO', '', repoNames_1.join(','), 'REPOSITORY.DELETION_SUMMARY_REPO', 2 /* DELETE_CANCEL */);
            });
        }
    };
    RepositoryListviewComponent.prototype.getTagInfo = function (repoName) {
        var _this = this;
        // this.signedNameArr = [];
        this.signedCon[repoName] = [];
        return toPromise(this.tagService
            .getTags(repoName))
            .then(function (items) {
            items.forEach(function (t) {
                if (t.signature !== null) {
                    _this.signedCon[repoName].push(t.name);
                }
            });
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryListviewComponent.prototype.signedDataSet = function (repoName) {
        var signature = '';
        if (this.signedCon[repoName].length === 0) {
            this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO', 2 /* DELETE_CANCEL */);
            return;
        }
        signature = this.signedCon[repoName].join(',');
        this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO_SIGNED', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO_SIGNED', 3 /* CLOSE */);
    };
    RepositoryListviewComponent.prototype.confirmationDialogSet = function (summaryTitle, signature, repoName, summaryKey, button) {
        var _this = this;
        this.translateService.get(summaryKey, {
            repoName: repoName,
            signedImages: signature,
        })
            .subscribe(function (res) {
            summaryKey = res;
            var message = new ConfirmationMessage(summaryTitle, summaryKey, repoName, repoName, 7 /* REPOSITORY */, button);
            _this.confirmationDialog.open(message);
            var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
            setTimeout(function () { return clearInterval(hnd); }, 5000);
        });
    };
    RepositoryListviewComponent.prototype.selectedChange = function () {
        var _this = this;
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 2000);
    };
    RepositoryListviewComponent.prototype.refresh = function () {
        this.doSearchRepoNames('');
    };
    RepositoryListviewComponent.prototype.clrLoad = function (state) {
        var _this = this;
        this.selectedRow = [];
        // Keep it for future filtering and sorting
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        // Pagination
        var params = new RequestQueryParams();
        params.set("page", '' + pageNumber);
        params.set("page_size", '' + this.pageSize);
        this.loading = true;
        toPromise(this.repositoryService.getRepositories(this.projectId, this.lastFilteredRepoName, params))
            .then(function (repo) {
            _this.totalCount = repo.metadata.xTotalCount;
            _this.repositories = repo.data;
            _this.signedCon = {};
            // Do filtering and sorting
            _this.repositories = doFiltering(_this.repositories, state);
            _this.repositories = doSorting(_this.repositories, state);
            _this.loading = false;
        })
            .catch(function (error) {
            _this.loading = false;
            _this.errorHandler.error(error);
        });
        // Force refresh view
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    RepositoryListviewComponent.prototype.getStateAfterDeletion = function () {
        var total = this.totalCount - 1;
        if (total <= 0) {
            return null;
        }
        var totalPages = Math.ceil(total / this.pageSize);
        var targetPageNumber = this.currentPage;
        if (this.currentPage > totalPages) {
            targetPageNumber = totalPages; // Should == currentPage -1
        }
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = (targetPageNumber - 1) * this.pageSize;
        st.page.to = targetPageNumber * this.pageSize - 1;
        return st;
    };
    RepositoryListviewComponent.prototype.watchRepoClickEvt = function (repo) {
        this.repoClickEvent.emit(repo);
    };
    return RepositoryListviewComponent;
}());
export { RepositoryListviewComponent };
RepositoryListviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-repository-listview',
                template: REPOSITORY_LISTVIEW_TEMPLATE,
                styles: [REPOSITORY_LISTVIEW_STYLE],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
RepositoryListviewComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: RepositoryService, },
    { type: SystemInfoService, },
    { type: TagService, },
    { type: ChangeDetectorRef, },
    { type: Router, },
]; };
RepositoryListviewComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'projectName': [{ type: Input },],
    'urlPrefix': [{ type: Input },],
    'hasSignedIn': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'repoClickEvent': [{ type: Output },],
    'confirmationDialog': [{ type: ViewChild, args: ['confirmationDialog',] },],
};
//# sourceMappingURL=repository-listview.component.js.map