import { Component, Input, Output, ViewChild, ChangeDetectionStrategy, ChangeDetectorRef, EventEmitter } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { REPOSITORY_STACKVIEW_TEMPLATE } from './repository-stackview.component.html';
import { REPOSITORY_STACKVIEW_STYLES } from './repository-stackview.component.css';
import { SystemInfoService, RepositoryService, RequestQueryParams } from '../service/index';
import { ErrorHandler } from '../error-handler/error-handler';
import { toPromise, CustomComparator } from '../utils';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { DEFAULT_PAGE_SIZE, calculatePage, doFiltering, doSorting } from '../utils';
import { TagService } from '../service/index';
var RepositoryStackviewComponent = (function () {
    function RepositoryStackviewComponent(errorHandler, translateService, repositoryService, systemInfoService, translate, tagService, ref) {
        this.errorHandler = errorHandler;
        this.translateService = translateService;
        this.repositoryService = repositoryService;
        this.systemInfoService = systemInfoService;
        this.translate = translate;
        this.tagService = tagService;
        this.ref = ref;
        this.signedCon = {};
        this.projectName = "unknown";
        this.tagClickEvent = new EventEmitter();
        this.loading = true;
        this.pullCountComparator = new CustomComparator('pull_count', 'number');
        this.tagsCountComparator = new CustomComparator('tags_count', 'number');
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.currentPage = 1;
        this.totalCount = 0;
    }
    Object.defineProperty(RepositoryStackviewComponent.prototype, "registryUrl", {
        get: function () {
            return this.systemInfo ? this.systemInfo.registry_url : "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryStackviewComponent.prototype, "withNotary", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_notary : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryStackviewComponent.prototype, "withClair", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_clair : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryStackviewComponent.prototype, "isClairDBReady", {
        get: function () {
            return this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.overall_last_update > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryStackviewComponent.prototype, "showDBStatusWarning", {
        get: function () {
            return this.withClair && !this.isClairDBReady;
        },
        enumerable: true,
        configurable: true
    });
    RepositoryStackviewComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 7 /* REPOSITORY */ &&
            message.state === 1 /* CONFIRMED */) {
            var repoName = message.data;
            toPromise(this.repositoryService
                .deleteRepository(repoName))
                .then(function (response) {
                _this.refresh();
                var st = _this.getStateAfterDeletion();
                if (!st) {
                    _this.refresh();
                }
                else {
                    _this.clrLoad(st);
                }
                _this.translateService.get('REPOSITORY.DELETED_REPO_SUCCESS')
                    .subscribe(function (res) { return _this.errorHandler.info(res); });
            }).catch(function (error) {
                if (error.status === "412") {
                    _this.translateService.get('REPOSITORY.TAGS_SIGNED')
                        .subscribe(function (res) { return _this.errorHandler.info(res); });
                    return;
                }
                _this.errorHandler.error(error);
            });
        }
    };
    RepositoryStackviewComponent.prototype.ngOnChanges = function (changes) {
        if (changes['projectId'] && changes['projectId'].currentValue) {
            this.refresh();
        }
    };
    RepositoryStackviewComponent.prototype.ngOnInit = function () {
        var _this = this;
        //Get system info for tag views
        toPromise(this.systemInfoService.getSystemInfo())
            .then(function (systemInfo) { return _this.systemInfo = systemInfo; })
            .catch(function (error) { return _this.errorHandler.error(error); });
        this.lastFilteredRepoName = '';
    };
    RepositoryStackviewComponent.prototype.doSearchRepoNames = function (repoName) {
        this.lastFilteredRepoName = repoName;
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        this.clrLoad(st);
    };
    RepositoryStackviewComponent.prototype.saveSignatures = function (event) {
        Object.assign(this.signedCon, event);
    };
    RepositoryStackviewComponent.prototype.deleteRepo = function (repoName) {
        var _this = this;
        if (this.signedCon[repoName]) {
            this.signedDataSet(repoName);
        }
        else {
            this.getTagInfo(repoName).then(function () {
                _this.signedDataSet(repoName);
            });
        }
    };
    RepositoryStackviewComponent.prototype.getTagInfo = function (repoName) {
        var _this = this;
        // this.signedNameArr = [];
        this.signedCon[repoName] = [];
        return toPromise(this.tagService
            .getTags(repoName))
            .then(function (items) {
            items.forEach(function (t) {
                if (t.signature !== null) {
                    _this.signedCon[repoName].push(t.name);
                }
            });
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryStackviewComponent.prototype.signedDataSet = function (repoName) {
        var signature = '';
        if (this.signedCon[repoName].length === 0) {
            this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO', 2 /* DELETE_CANCEL */);
            return;
        }
        signature = this.signedCon[repoName].join(',');
        this.confirmationDialogSet('REPOSITORY.DELETION_TITLE_REPO_SIGNED', signature, repoName, 'REPOSITORY.DELETION_SUMMARY_REPO_SIGNED', 3 /* CLOSE */);
    };
    RepositoryStackviewComponent.prototype.confirmationDialogSet = function (summaryTitle, signature, repoName, summaryKey, button) {
        var _this = this;
        this.translate.get(summaryKey, {
            repoName: repoName,
            signedImages: signature,
        })
            .subscribe(function (res) {
            summaryKey = res;
            var message = new ConfirmationMessage(summaryTitle, summaryKey, repoName, repoName, 7 /* REPOSITORY */, button);
            _this.confirmationDialog.open(message);
            var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
            setTimeout(function () { return clearInterval(hnd); }, 5000);
        });
    };
    RepositoryStackviewComponent.prototype.refresh = function () {
        this.doSearchRepoNames("");
    };
    RepositoryStackviewComponent.prototype.watchTagClickEvt = function (tagClickEvt) {
        this.tagClickEvent.emit(tagClickEvt);
    };
    RepositoryStackviewComponent.prototype.clrLoad = function (state) {
        var _this = this;
        //Keep it for future filtering and sorting
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        //Pagination
        var params = new RequestQueryParams();
        params.set("page", '' + pageNumber);
        params.set("page_size", '' + this.pageSize);
        this.loading = true;
        toPromise(this.repositoryService.getRepositories(this.projectId, this.lastFilteredRepoName, params))
            .then(function (repo) {
            _this.totalCount = repo.metadata.xTotalCount;
            _this.repositories = repo.data;
            _this.signedCon = {};
            //Do filtering and sorting
            _this.repositories = doFiltering(_this.repositories, state);
            _this.repositories = doSorting(_this.repositories, state);
            _this.loading = false;
        })
            .catch(function (error) {
            _this.loading = false;
            _this.errorHandler.error(error);
        });
        //Force refresh view
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    RepositoryStackviewComponent.prototype.getStateAfterDeletion = function () {
        var total = this.totalCount - 1;
        if (total <= 0) {
            return null;
        }
        var totalPages = Math.ceil(total / this.pageSize);
        var targetPageNumber = this.currentPage;
        if (this.currentPage > totalPages) {
            targetPageNumber = totalPages; //Should == currentPage -1
        }
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = (targetPageNumber - 1) * this.pageSize;
        st.page.to = targetPageNumber * this.pageSize - 1;
        return st;
    };
    return RepositoryStackviewComponent;
}());
export { RepositoryStackviewComponent };
RepositoryStackviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-repository-stackview',
                template: REPOSITORY_STACKVIEW_TEMPLATE,
                styles: [REPOSITORY_STACKVIEW_STYLES],
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
RepositoryStackviewComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: RepositoryService, },
    { type: SystemInfoService, },
    { type: TranslateService, },
    { type: TagService, },
    { type: ChangeDetectorRef, },
]; };
RepositoryStackviewComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'projectName': [{ type: Input },],
    'hasSignedIn': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'tagClickEvent': [{ type: Output },],
    'confirmationDialog': [{ type: ViewChild, args: ['confirmationDialog',] },],
};
//# sourceMappingURL=repository-stackview.component.js.map