// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { RepositoryService } from '../service/repository.service';
import { SystemInfoService, TagService } from '../service/index';
import { TranslateService } from '@ngx-translate/core';
import { ErrorHandler } from '../error-handler/error-handler';
import { ConfirmationMessage } from '../confirmation-dialog/confirmation-message';
import { toPromise } from '../utils';
import { REPOSITORY_TEMPLATE } from './repository.component.html';
import { REPOSITORY_STYLE } from './repository.component.css';
var TabLinkContentMap = {
    'repo-info': 'info',
    'repo-image': 'image'
};
var RepositoryComponent = (function () {
    function RepositoryComponent(errorHandler, repositoryService, systemInfoService, tagService, translate) {
        this.errorHandler = errorHandler;
        this.repositoryService = repositoryService;
        this.systemInfoService = systemInfoService;
        this.tagService = tagService;
        this.translate = translate;
        this.signedCon = {};
        this.tagClickEvent = new EventEmitter();
        this.backEvt = new EventEmitter();
        this.onGoing = false;
        this.editing = false;
        this.inProgress = true;
        this.currentTabID = 'repo-image';
    }
    Object.defineProperty(RepositoryComponent.prototype, "registryUrl", {
        get: function () {
            return this.systemInfo ? this.systemInfo.registry_url : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryComponent.prototype, "withNotary", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_notary : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryComponent.prototype, "withClair", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_clair : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RepositoryComponent.prototype, "withAdmiral", {
        get: function () {
            return this.systemInfo ? this.systemInfo.with_admiral : false;
        },
        enumerable: true,
        configurable: true
    });
    RepositoryComponent.prototype.ngOnInit = function () {
        if (!this.projectId) {
            this.errorHandler.error('Project ID cannot be unset.');
            return;
        }
        this.retrieve();
        this.inProgress = false;
    };
    RepositoryComponent.prototype.retrieve = function (state) {
        var _this = this;
        toPromise(this.repositoryService.getRepositories(this.projectId, this.repoName))
            .then(function (response) {
            if (response.metadata.xTotalCount > 0) {
                _this.orgImageInfo = response.data[0].description;
                _this.imageInfo = response.data[0].description;
            }
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
        toPromise(this.systemInfoService.getSystemInfo())
            .then(function (systemInfo) { return _this.systemInfo = systemInfo; })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryComponent.prototype.saveSignatures = function (event) {
        Object.assign(this.signedCon, event);
    };
    RepositoryComponent.prototype.refresh = function () {
        this.retrieve();
    };
    RepositoryComponent.prototype.watchTagClickEvt = function (tagClickEvt) {
        this.tagClickEvent.emit(tagClickEvt);
    };
    RepositoryComponent.prototype.isCurrentTabLink = function (tabID) {
        return this.currentTabID === tabID;
    };
    RepositoryComponent.prototype.isCurrentTabContent = function (ContentID) {
        return TabLinkContentMap[this.currentTabID] === ContentID;
    };
    RepositoryComponent.prototype.tabLinkClick = function (tabID) {
        this.currentTabID = tabID;
    };
    RepositoryComponent.prototype.getTagInfo = function (repoName) {
        var _this = this;
        // this.signedNameArr = [];
        this.signedCon[repoName] = [];
        return toPromise(this.tagService
            .getTags(repoName))
            .then(function (items) {
            items.forEach(function (t) {
                if (t.signature !== null) {
                    _this.signedCon[repoName].push(t.name);
                }
            });
        })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    RepositoryComponent.prototype.goBack = function () {
        this.backEvt.emit(this.projectId);
    };
    RepositoryComponent.prototype.hasChanges = function () {
        return this.imageInfo !== this.orgImageInfo;
    };
    RepositoryComponent.prototype.reset = function () {
        this.imageInfo = this.orgImageInfo;
    };
    RepositoryComponent.prototype.hasInfo = function () {
        return this.imageInfo && this.imageInfo.length > 0;
    };
    RepositoryComponent.prototype.editInfo = function () {
        this.editing = true;
    };
    RepositoryComponent.prototype.saveInfo = function () {
        var _this = this;
        if (!this.hasChanges()) {
            return;
        }
        this.onGoing = true;
        toPromise(this.repositoryService.updateRepositoryDescription(this.repoName, this.imageInfo))
            .then(function () {
            _this.onGoing = false;
            _this.translate.get('CONFIG.SAVE_SUCCESS').subscribe(function (res) {
                _this.errorHandler.info(res);
            });
            _this.editing = false;
            _this.refresh();
        })
            .catch(function (error) {
            _this.onGoing = false;
            _this.errorHandler.error(error);
        });
    };
    RepositoryComponent.prototype.cancelInfo = function () {
        var msg = new ConfirmationMessage('CONFIG.CONFIRM_TITLE', 'CONFIG.CONFIRM_SUMMARY', '', {}, 9 /* CONFIG */);
        this.confirmationDlg.open(msg);
    };
    RepositoryComponent.prototype.confirmCancel = function (ack) {
        this.editing = false;
        if (ack && ack.source === 9 /* CONFIG */ &&
            ack.state === 1 /* CONFIRMED */) {
            this.reset();
        }
    };
    return RepositoryComponent;
}());
export { RepositoryComponent };
RepositoryComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-repository',
                template: REPOSITORY_TEMPLATE,
                styles: [REPOSITORY_STYLE]
            },] },
];
/** @nocollapse */
RepositoryComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: RepositoryService, },
    { type: SystemInfoService, },
    { type: TagService, },
    { type: TranslateService, },
]; };
RepositoryComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'repoName': [{ type: Input },],
    'hasSignedIn': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'isGuest': [{ type: Input },],
    'tagClickEvent': [{ type: Output },],
    'backEvt': [{ type: Output },],
    'confirmationDlg': [{ type: ViewChild, args: ['confirmationDialog',] },],
};
//# sourceMappingURL=repository.component.js.map