// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import { Component, ViewChild, Input, Output, EventEmitter, ChangeDetectorRef } from "@angular/core";
import { TagService, VulnerabilitySeverity, RequestQueryParams } from "../service/index";
import { ErrorHandler } from "../error-handler/error-handler";
import { ChannelService } from "../channel/index";
import { ConfirmationMessage } from "../confirmation-dialog/confirmation-message";
import { TAG_TEMPLATE } from "./tag.component.html";
import { TAG_STYLE } from "./tag.component.css";
import { toPromise, CustomComparator, calculatePage, doFiltering, doSorting, VULNERABILITY_SCAN_STATUS, DEFAULT_PAGE_SIZE, clone, } from "../utils";
import { TranslateService } from "@ngx-translate/core";
import { BatchInfo, BathInfoChanges } from "../confirmation-dialog/confirmation-batch-message";
import { Observable } from "rxjs/Observable";
import { LabelService } from "../service/label.service";
import { Subject } from "rxjs/Subject";
var TagComponent = (function () {
    function TagComponent(errorHandler, tagService, labelService, translateService, ref, channel) {
        this.errorHandler = errorHandler;
        this.tagService = tagService;
        this.labelService = labelService;
        this.translateService = translateService;
        this.ref = ref;
        this.channel = channel;
        this.signedCon = {};
        this.refreshRepo = new EventEmitter();
        this.tagClickEvent = new EventEmitter();
        this.signatureOutput = new EventEmitter();
        this.staticBackdrop = true;
        this.closable = false;
        this.batchDelectionInfos = [];
        this.createdComparator = new CustomComparator("created", "date");
        this.loading = false;
        this.copyFailed = false;
        this.selectedRow = [];
        this.imageLabels = [];
        this.imageStickLabels = [];
        this.imageFilterLabels = [];
        this.labelListOpen = false;
        this.labelNameFilter = new Subject();
        this.stickLabelNameFilter = new Subject();
        this.stickName = '';
        this.filterName = '';
        this.initFilter = {
            name: '',
            description: '',
            color: '',
            scope: '',
            project_id: 0,
        };
        this.filterOneLabel = this.initFilter;
        this.pageSize = DEFAULT_PAGE_SIZE;
        this.currentPage = 1;
        this.totalCount = 0;
    }
    TagComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (!this.projectId) {
            this.errorHandler.error("Project ID cannot be unset.");
            return;
        }
        if (!this.repoName) {
            this.errorHandler.error("Repo name cannot be unset.");
            return;
        }
        this.retrieve();
        this.lastFilteredTagName = '';
        this.labelNameFilter
            .debounceTime(500)
            .distinctUntilChanged()
            .subscribe(function (name) {
            if (_this.filterName.length) {
                _this.filterOnGoing = true;
                _this.imageFilterLabels.forEach(function (data) {
                    if (data.label.name.indexOf(_this.filterName) !== -1) {
                        data.show = true;
                    }
                    else {
                        data.show = false;
                    }
                });
                setTimeout(function () {
                    setInterval(function () { return _this.ref.markForCheck(); }, 200);
                }, 1000);
            }
        });
        this.stickLabelNameFilter
            .debounceTime(500)
            .distinctUntilChanged()
            .subscribe(function (name) {
            if (_this.stickName.length) {
                _this.filterOnGoing = true;
                _this.imageStickLabels.forEach(function (data) {
                    if (data.label.name.indexOf(_this.stickName) !== -1) {
                        data.show = true;
                    }
                    else {
                        data.show = false;
                    }
                });
                setTimeout(function () {
                    setInterval(function () { return _this.ref.markForCheck(); }, 200);
                }, 1000);
            }
        });
    };
    TagComponent.prototype.ngAfterViewInit = function () {
        if (!this.withAdmiral) {
            this.getAllLabels();
        }
    };
    Object.defineProperty(TagComponent.prototype, "filterLabelPieceWidth", {
        get: function () {
            var len = this.lastFilteredTagName.length ? this.lastFilteredTagName.length * 6 + 60 : 115;
            return len > 210 ? 210 : len;
        },
        enumerable: true,
        configurable: true
    });
    TagComponent.prototype.doSearchTagNames = function (tagName) {
        this.lastFilteredTagName = tagName;
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        var selectedLab = this.imageFilterLabels.find(function (label) { return label.iconsShow === true; });
        if (selectedLab) {
            st.filters = [{ property: 'name', value: this.lastFilteredTagName }, { property: 'labels.id', value: selectedLab.label.id }];
        }
        else {
            st.filters = [{ property: 'name', value: this.lastFilteredTagName }];
        }
        this.clrLoad(st);
    };
    TagComponent.prototype.clrLoad = function (state) {
        var _this = this;
        this.selectedRow = [];
        // Keep it for future filtering and sorting
        this.currentState = state;
        var pageNumber = calculatePage(state);
        if (pageNumber <= 0) {
            pageNumber = 1;
        }
        // Pagination
        var params = new RequestQueryParams();
        params.set("page", "" + pageNumber);
        params.set("page_size", "" + this.pageSize);
        this.loading = true;
        toPromise(this.tagService.getTags(this.repoName, params))
            .then(function (tags) {
            _this.signedCon = {};
            // Do filtering and sorting
            _this.tags = doFiltering(tags, state);
            _this.tags = doSorting(_this.tags, state);
            _this.loading = false;
        })
            .catch(function (error) {
            _this.loading = false;
            _this.errorHandler.error(error);
        });
        // Force refresh view
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    TagComponent.prototype.refresh = function () {
        this.doSearchTagNames("");
    };
    TagComponent.prototype.getAllLabels = function () {
        var _this = this;
        toPromise(this.labelService.getGLabels()).then(function (res) {
            if (res.length) {
                res.forEach(function (data) {
                    _this.imageLabels.push({ 'iconsShow': false, 'label': data, 'show': true });
                });
            }
            toPromise(_this.labelService.getPLabels(_this.projectId)).then(function (res1) {
                if (res1.length) {
                    res1.forEach(function (data) {
                        _this.imageLabels.push({ 'iconsShow': false, 'label': data, 'show': true });
                    });
                }
                _this.imageFilterLabels = clone(_this.imageLabels);
                _this.imageStickLabels = clone(_this.imageLabels);
            }).catch(function (error) {
                _this.errorHandler.error(error);
            });
        }).catch(function (error) {
            _this.errorHandler.error(error);
        });
    };
    TagComponent.prototype.labelSelectedChange = function (tag) {
        var _this = this;
        if (tag && tag[0].labels) {
            this.imageStickLabels.forEach(function (data) {
                data.iconsShow = false;
                data.show = true;
            });
            if (tag[0].labels.length) {
                tag[0].labels.forEach(function (labelInfo) {
                    var findedLabel = _this.imageStickLabels.find(function (data) { return labelInfo.id === data['label'].id; });
                    _this.imageStickLabels.splice(_this.imageStickLabels.indexOf(findedLabel), 1);
                    _this.imageStickLabels.unshift(findedLabel);
                    findedLabel.iconsShow = true;
                });
            }
        }
    };
    TagComponent.prototype.addLabels = function (tag) {
        this.labelListOpen = true;
        this.selectedTag = tag;
        this.stickName = '';
        this.labelSelectedChange(tag);
    };
    TagComponent.prototype.stickLabel = function (labelInfo) {
        if (labelInfo && !labelInfo.iconsShow) {
            this.selectLabel(labelInfo);
        }
        if (labelInfo && labelInfo.iconsShow) {
            this.unSelectLabel(labelInfo);
        }
    };
    TagComponent.prototype.selectLabel = function (labelInfo) {
        var _this = this;
        if (!this.inprogress) {
            this.inprogress = true;
            var labelId = labelInfo.label.id;
            this.selectedRow = this.selectedTag;
            toPromise(this.tagService.addLabelToImages(this.repoName, this.selectedRow[0].name, labelId)).then(function (res) {
                _this.refresh();
                // set the selected label in front
                _this.imageStickLabels.splice(_this.imageStickLabels.indexOf(labelInfo), 1);
                _this.imageStickLabels.unshift(labelInfo);
                labelInfo.iconsShow = true;
                _this.inprogress = false;
            }).catch(function (err) {
                _this.inprogress = false;
                _this.errorHandler.error(err);
            });
        }
    };
    TagComponent.prototype.unSelectLabel = function (labelInfo) {
        var _this = this;
        if (!this.inprogress) {
            this.inprogress = true;
            var labelId = labelInfo.label.id;
            this.selectedRow = this.selectedTag;
            toPromise(this.tagService.deleteLabelToImages(this.repoName, this.selectedRow[0].name, labelId)).then(function (res) {
                _this.refresh();
                // insert the unselected label to groups with the same icons
                _this.sortOperation(_this.imageStickLabels, labelInfo);
                labelInfo.iconsShow = false;
                _this.inprogress = false;
            }).catch(function (err) {
                _this.inprogress = false;
                _this.errorHandler.error(err);
            });
        }
    };
    TagComponent.prototype.rightFilterLabel = function (labelInfo) {
        if (labelInfo) {
            if (!labelInfo.iconsShow) {
                this.filterLabel(labelInfo);
            }
            else {
                this.unFilterLabel(labelInfo);
            }
        }
    };
    TagComponent.prototype.filterLabel = function (labelInfo) {
        var _this = this;
        var labelName = labelInfo.label.name;
        var labelId = labelInfo.label.id;
        // insert the unselected label to groups with the same icons
        var preLabelInfo = this.imageFilterLabels.find(function (data) { return data.label.id === _this.filterOneLabel.id; });
        if (preLabelInfo) {
            this.sortOperation(this.imageFilterLabels, preLabelInfo);
        }
        this.imageFilterLabels.filter(function (data) {
            if (data.label.id !== labelId) {
                data.iconsShow = false;
            }
            else {
                data.iconsShow = true;
            }
        });
        this.imageFilterLabels.splice(this.imageFilterLabels.indexOf(labelInfo), 1);
        this.imageFilterLabels.unshift(labelInfo);
        this.filterOneLabel = labelInfo.label;
        // reload data
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        if (this.lastFilteredTagName) {
            st.filters = [{ property: 'name', value: this.lastFilteredTagName }, { property: 'labels.id', value: labelId }];
        }
        else {
            st.filters = [{ property: 'labels.id', value: labelId }];
        }
        this.clrLoad(st);
    };
    TagComponent.prototype.unFilterLabel = function (labelInfo) {
        // insert the unselected label to groups with the same icons
        this.sortOperation(this.imageFilterLabels, labelInfo);
        this.filterOneLabel = this.initFilter;
        labelInfo.iconsShow = false;
        // reload data
        this.currentPage = 1;
        var st = this.currentState;
        if (!st) {
            st = { page: {} };
        }
        st.page.size = this.pageSize;
        st.page.from = 0;
        st.page.to = this.pageSize - 1;
        if (this.lastFilteredTagName) {
            st.filters = [{ property: 'name', value: this.lastFilteredTagName }];
        }
        else {
            st.filters = [];
        }
        this.clrLoad(st);
    };
    TagComponent.prototype.handleInputFilter = function () {
        if (this.filterName.length) {
            this.labelNameFilter.next(this.filterName);
        }
        else {
            this.imageFilterLabels.every(function (data) { return data.show = true; });
        }
    };
    TagComponent.prototype.handleStickInputFilter = function () {
        if (this.stickName.length) {
            this.stickLabelNameFilter.next(this.stickName);
        }
        else {
            this.imageStickLabels.every(function (data) { return data.show = true; });
        }
    };
    // insert the unselected label to groups with the same icons
    TagComponent.prototype.sortOperation = function (labelList, labelInfo) {
        labelList.some(function (data, i) {
            if (!data.iconsShow) {
                if (data.label.scope === labelInfo.label.scope) {
                    labelList.splice(i, 0, labelInfo);
                    labelList.splice(labelList.indexOf(labelInfo, 0), 1);
                    return true;
                }
                if (data.label.scope !== labelInfo.label.scope && i === labelList.length - 1) {
                    labelList.push(labelInfo);
                    labelList.splice(labelList.indexOf(labelInfo), 1);
                    return true;
                }
            }
        });
    };
    TagComponent.prototype.retrieve = function () {
        var _this = this;
        this.tags = [];
        var signatures = [];
        this.loading = true;
        toPromise(this.tagService
            .getTags(this.repoName))
            .then(function (items) {
            // To keep easy use for vulnerability bar
            items.forEach(function (t) {
                if (!t.scan_overview) {
                    t.scan_overview = {
                        scan_status: VULNERABILITY_SCAN_STATUS.stopped,
                        severity: VulnerabilitySeverity.UNKNOWN,
                        update_time: new Date(),
                        components: {
                            total: 0,
                            summary: []
                        }
                    };
                }
                if (t.signature !== null) {
                    signatures.push(t.name);
                }
            });
            _this.tags = items;
            var signedName = {};
            signedName[_this.repoName] = signatures;
            _this.signatureOutput.emit(signedName);
            _this.loading = false;
            if (_this.tags && _this.tags.length === 0) {
                _this.refreshRepo.emit(true);
            }
        })
            .catch(function (error) {
            _this.errorHandler.error(error);
            _this.loading = false;
        });
        var hnd = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () { return clearInterval(hnd); }, 5000);
    };
    TagComponent.prototype.sizeTransform = function (tagSize) {
        var size = Number.parseInt(tagSize);
        if (Math.pow(1024, 1) <= size && size < Math.pow(1024, 2)) {
            return (size / Math.pow(1024, 1)).toFixed(2) + "KB";
        }
        else if (Math.pow(1024, 2) <= size && size < Math.pow(1024, 3)) {
            return (size / Math.pow(1024, 2)).toFixed(2) + "MB";
        }
        else if (Math.pow(1024, 3) <= size && size < Math.pow(1024, 4)) {
            return (size / Math.pow(1024, 3)).toFixed(2) + "GB";
        }
        else {
            return size + "B";
        }
    };
    TagComponent.prototype.deleteTags = function (tags) {
        var _this = this;
        if (tags && tags.length) {
            var tagNames_1 = [];
            this.batchDelectionInfos = [];
            tags.forEach(function (tag) {
                tagNames_1.push(tag.name);
                var initBatchMessage = new BatchInfo();
                initBatchMessage.name = tag.name;
                _this.batchDelectionInfos.push(initBatchMessage);
            });
            var titleKey = void 0, summaryKey = void 0, content = void 0, buttons = void 0;
            titleKey = "REPOSITORY.DELETION_TITLE_TAG";
            summaryKey = "REPOSITORY.DELETION_SUMMARY_TAG";
            buttons = 2 /* DELETE_CANCEL */;
            content = tagNames_1.join(" , ");
            var message = new ConfirmationMessage(titleKey, summaryKey, content, tags, 8 /* TAG */, buttons);
            this.confirmationDialog.open(message);
        }
    };
    TagComponent.prototype.confirmDeletion = function (message) {
        var _this = this;
        if (message &&
            message.source === 8 /* TAG */
            && message.state === 1 /* CONFIRMED */) {
            var tags = message.data;
            if (tags && tags.length) {
                var promiseLists_1 = [];
                tags.forEach(function (tag) {
                    promiseLists_1.push(_this.delOperate(tag.signature, tag.name));
                });
                Promise.all(promiseLists_1).then(function (item) {
                    _this.selectedRow = [];
                    _this.retrieve();
                });
            }
        }
    };
    TagComponent.prototype.delOperate = function (signature, name) {
        var _this = this;
        var findedList = this.batchDelectionInfos.find(function (data) { return data.name === name; });
        if (signature) {
            Observable.forkJoin(this.translateService.get("BATCH.DELETED_FAILURE"), this.translateService.get("REPOSITORY.DELETION_SUMMARY_TAG_DENIED")).subscribe(function (res) {
                var wrongInfo = res[1] + "notary -s https://" + _this.registryUrl + ":4443 -d ~/.docker/trust remove -p " + _this.registryUrl + "/" + _this.repoName + " " + name;
                findedList = BathInfoChanges(findedList, res[0], false, true, wrongInfo);
            });
        }
        else {
            return toPromise(this.tagService
                .deleteTag(this.repoName, name))
                .then(function (response) {
                _this.translateService.get("BATCH.DELETED_SUCCESS")
                    .subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res);
                });
            }).catch(function (error) {
                if (error.status === 503) {
                    Observable.forkJoin(_this.translateService.get('BATCH.DELETED_FAILURE'), _this.translateService.get('REPOSITORY.TAGS_NO_DELETE')).subscribe(function (res) {
                        findedList = BathInfoChanges(findedList, res[0], false, true, res[1]);
                    });
                    return;
                }
                _this.translateService.get("BATCH.DELETED_FAILURE").subscribe(function (res) {
                    findedList = BathInfoChanges(findedList, res, false, true);
                });
            });
        }
    };
    TagComponent.prototype.showDigestId = function (tag) {
        if (tag && (tag.length === 1)) {
            this.manifestInfoTitle = "REPOSITORY.COPY_DIGEST_ID";
            this.digestId = tag[0].digest;
            this.showTagManifestOpened = true;
            this.copyFailed = false;
        }
    };
    TagComponent.prototype.onTagClick = function (tag) {
        if (tag) {
            var evt = {
                project_id: this.projectId,
                repository_name: this.repoName,
                tag_name: tag.name
            };
            this.tagClickEvent.emit(evt);
        }
    };
    TagComponent.prototype.onSuccess = function ($event) {
        this.copyFailed = false;
        // Directly close dialog
        this.showTagManifestOpened = false;
    };
    TagComponent.prototype.onError = function ($event) {
        // Show error
        this.copyFailed = true;
        // Select all text
        if (this.textInput) {
            this.textInput.nativeElement.select();
        }
    };
    // Get vulnerability scanning status
    TagComponent.prototype.scanStatus = function (t) {
        if (t && t.scan_overview && t.scan_overview.scan_status) {
            return t.scan_overview.scan_status;
        }
        return VULNERABILITY_SCAN_STATUS.unknown;
    };
    TagComponent.prototype.existObservablePackage = function (t) {
        return t.scan_overview &&
            t.scan_overview.components &&
            t.scan_overview.components.total &&
            t.scan_overview.components.total > 0 ? true : false;
    };
    // Whether show the 'scan now' menu
    TagComponent.prototype.canScanNow = function (t) {
        if (!this.withClair) {
            return false;
        }
        if (!this.hasProjectAdminRole) {
            return false;
        }
        var st = this.scanStatus(t[0]);
        return st !== VULNERABILITY_SCAN_STATUS.pending &&
            st !== VULNERABILITY_SCAN_STATUS.running;
    };
    // Trigger scan
    TagComponent.prototype.scanNow = function (t) {
        var _this = this;
        if (t && t.length) {
            t.forEach(function (data) {
                var tagId = data.name;
                _this.channel.publishScanEvent(_this.repoName + "/" + tagId);
            });
        }
    };
    // pull command
    TagComponent.prototype.onCpError = function ($event) {
        this.copyInput.setPullCommendShow();
    };
    return TagComponent;
}());
export { TagComponent };
TagComponent.decorators = [
    { type: Component, args: [{
                selector: "hbr-tag",
                template: TAG_TEMPLATE,
                styles: [TAG_STYLE],
            },] },
];
/** @nocollapse */
TagComponent.ctorParameters = function () { return [
    { type: ErrorHandler, },
    { type: TagService, },
    { type: LabelService, },
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
    { type: ChannelService, },
]; };
TagComponent.propDecorators = {
    'projectId': [{ type: Input },],
    'repoName': [{ type: Input },],
    'isEmbedded': [{ type: Input },],
    'hasSignedIn': [{ type: Input },],
    'hasProjectAdminRole': [{ type: Input },],
    'isGuest': [{ type: Input },],
    'registryUrl': [{ type: Input },],
    'withNotary': [{ type: Input },],
    'withClair': [{ type: Input },],
    'withAdmiral': [{ type: Input },],
    'refreshRepo': [{ type: Output },],
    'tagClickEvent': [{ type: Output },],
    'signatureOutput': [{ type: Output },],
    'confirmationDialog': [{ type: ViewChild, args: ['confirmationDialog',] },],
    'textInput': [{ type: ViewChild, args: ["digestTarget",] },],
    'copyInput': [{ type: ViewChild, args: ["copyInput",] },],
};
//# sourceMappingURL=tag.component.js.map