import { Component, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { VULNERABILITY_CONFIG_HTML, VULNERABILITY_CONFIG_STYLES } from './vulnerability-config.component.template';
import { ScanningResultService, SystemInfoService } from '../../service/index';
import { ErrorHandler } from '../../error-handler';
import { toPromise } from '../../utils';
import { TranslateService } from '@ngx-translate/core';
import { REGISTRY_CONFIG_STYLES } from '../registry-config.component.css';
var ONE_HOUR_SECONDS = 3600;
var ONE_DAY_SECONDS = 24 * ONE_HOUR_SECONDS;
var VulnerabilityConfigComponent = (function () {
    function VulnerabilityConfigComponent(scanningService, errorHandler, translate, systemInfoService) {
        this.scanningService = scanningService;
        this.errorHandler = errorHandler;
        this.translate = translate;
        this.systemInfoService = systemInfoService;
        this._localTime = new Date();
        this.onSubmitting = false;
        this.openState = false;
        this.configChange = new EventEmitter();
        this.showSubTitle = false;
        this.showScanningNamespaces = false;
    }
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "vulnerabilityConfig", {
        get: function () {
            return this.config;
        },
        set: function (cfg) {
            this.config = cfg;
            if (this.config.scan_all_policy &&
                this.config.scan_all_policy.value) {
                if (this.config.scan_all_policy.value.type === "daily") {
                    if (!this.config.scan_all_policy.value.parameter) {
                        this.config.scan_all_policy.value.parameter = {
                            daily_time: 0
                        };
                    }
                }
            }
            this.configChange.emit(this.config);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "scanAvailable", {
        get: function () {
            var dt = new Date();
            return !this.onSubmitting && (this.nextScanTime <= 0 || dt.getTime() > ((this.nextScanTime + 300) * 1000));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "nextScanTimestamp", {
        get: function () {
            return this.nextScanTime > 0 ? this.convertToLocalTime(this.nextScanTime) : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "nextScanTime", {
        get: function () {
            return this.systemInfo && this.systemInfo.next_scan_all ? this.systemInfo.next_scan_all : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "updatedTimestamp", {
        get: function () {
            if (this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.overall_last_update > 0) {
                return this.convertToLocalTime(this.systemInfo.clair_vulnerability_status.overall_last_update);
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "namespaceTimestamps", {
        get: function () {
            if (this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.details &&
                this.systemInfo.clair_vulnerability_status.details.length > 0) {
                return this.systemInfo.clair_vulnerability_status.details;
            }
            return [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "dailyTime", {
        //UTC time
        get: function () {
            if (!(this.config &&
                this.config.scan_all_policy &&
                this.config.scan_all_policy.value &&
                this.config.scan_all_policy.value.type === "daily")) {
                return "00:00";
            }
            var timeOffset = 0; //seconds
            if (this.config.scan_all_policy.value.parameter) {
                var daily_time = this.config.scan_all_policy.value.parameter.daily_time;
                if (daily_time && typeof daily_time === "number") {
                    timeOffset = +daily_time;
                }
            }
            //Convert to current time
            var timezoneOffset = this._localTime.getTimezoneOffset();
            //Local time
            timeOffset = timeOffset - timezoneOffset * 60;
            if (timeOffset < 0) {
                timeOffset = timeOffset + ONE_DAY_SECONDS;
            }
            if (timeOffset >= ONE_DAY_SECONDS) {
                timeOffset -= ONE_DAY_SECONDS;
            }
            //To time string
            var hours = Math.floor(timeOffset / ONE_HOUR_SECONDS);
            var minutes = Math.floor((timeOffset - hours * ONE_HOUR_SECONDS) / 60);
            var timeStr = "" + hours;
            if (hours < 10) {
                timeStr = "0" + timeStr;
            }
            if (minutes < 10) {
                timeStr += ":0";
            }
            else {
                timeStr += ":";
            }
            timeStr += minutes;
            return timeStr;
        },
        set: function (v) {
            if (!v || v === "") {
                return;
            }
            if (!(this.config &&
                this.config.scan_all_policy &&
                this.config.scan_all_policy.value &&
                this.config.scan_all_policy.value.type === "daily")) {
                return;
            }
            //Double confirm inner parameter existing.
            if (!this.config.scan_all_policy.value.parameter) {
                this.config.scan_all_policy.value.parameter = {
                    daily_time: 0
                };
            }
            var values = v.split(":");
            if (!values || values.length !== 2) {
                return;
            }
            var hours = +values[0];
            var minutes = +values[1];
            //Convert to UTC time
            var timezoneOffset = this._localTime.getTimezoneOffset();
            var utcTimes = hours * ONE_HOUR_SECONDS + minutes * 60;
            utcTimes += timezoneOffset * 60;
            if (utcTimes < 0) {
                utcTimes += ONE_DAY_SECONDS;
            }
            if (utcTimes >= ONE_DAY_SECONDS) {
                utcTimes -= ONE_DAY_SECONDS;
            }
            this.config.scan_all_policy.value.parameter.daily_time = utcTimes;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "scanningType", {
        //Scanning type
        get: function () {
            if (this.config &&
                this.config.scan_all_policy &&
                this.config.scan_all_policy.value) {
                return this.config.scan_all_policy.value.type;
            }
            else {
                //default
                return "none";
            }
        },
        set: function (v) {
            if (this.config &&
                this.config.scan_all_policy &&
                this.config.scan_all_policy.value) {
                var type = (v && v.trim() !== "") ? v : "none";
                this.config.scan_all_policy.value.type = type;
                if (type !== "daily") {
                    //No parameter
                    if (this.config.scan_all_policy.value.parameter) {
                        delete (this.config.scan_all_policy.value.parameter);
                    }
                }
                else {
                    //Has parameter
                    if (!this.config.scan_all_policy.value.parameter) {
                        this.config.scan_all_policy.value.parameter = {
                            daily_time: 0
                        };
                    }
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "editable", {
        get: function () {
            return this.vulnerabilityConfig &&
                this.vulnerabilityConfig.scan_all_policy &&
                this.vulnerabilityConfig.scan_all_policy.editable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "isValid", {
        get: function () {
            return this.systemSettingsForm && this.systemSettingsForm.valid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "showTimePicker", {
        get: function () {
            return this.vulnerabilityConfig &&
                this.vulnerabilityConfig.scan_all_policy &&
                this.vulnerabilityConfig.scan_all_policy.value &&
                this.vulnerabilityConfig.scan_all_policy.value.type === "daily";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VulnerabilityConfigComponent.prototype, "isClairDBFullyReady", {
        get: function () {
            return this.systemInfo &&
                this.systemInfo.clair_vulnerability_status &&
                this.systemInfo.clair_vulnerability_status.overall_last_update > 0;
        },
        enumerable: true,
        configurable: true
    });
    VulnerabilityConfigComponent.prototype.ngOnInit = function () {
        this.getSystemInfo();
    };
    VulnerabilityConfigComponent.prototype.convertToLocalTime = function (utcTime) {
        var dt = new Date();
        dt.setTime(utcTime * 1000);
        return dt;
    };
    VulnerabilityConfigComponent.prototype.scanNow = function () {
        var _this = this;
        if (this.onSubmitting) {
            return; //Aoid duplicated submitting
        }
        if (!this.scanAvailable) {
            return; //Aoid page hacking
        }
        this.onSubmitting = true;
        toPromise(this.scanningService.startScanningAll())
            .then(function () {
            _this.translate.get("CONFIG.SCANNING.TRIGGER_SCAN_ALL_SUCCESS").subscribe(function (res) {
                _this.errorHandler.info(res);
            });
            //Update system info
            _this.getSystemInfo().then(function () {
                _this.onSubmitting = false;
            }).catch(function () {
                _this.onSubmitting = false;
            });
        })
            .catch(function (error) {
            if (error && error.status && error.status === 412) {
                _this.translate.get("CONFIG.SCANNING.TRIGGER_SCAN_ALL_FAIL", { error: '' + error }).subscribe(function (res) {
                    _this.errorHandler.error(res);
                });
            }
            else {
                _this.errorHandler.error(error);
            }
            _this.onSubmitting = false;
        });
    };
    VulnerabilityConfigComponent.prototype.getSystemInfo = function () {
        var _this = this;
        return toPromise(this.systemInfoService.getSystemInfo())
            .then(function (info) { return _this.systemInfo = info; })
            .catch(function (error) { return _this.errorHandler.error(error); });
    };
    return VulnerabilityConfigComponent;
}());
export { VulnerabilityConfigComponent };
VulnerabilityConfigComponent.decorators = [
    { type: Component, args: [{
                selector: 'vulnerability-config',
                template: VULNERABILITY_CONFIG_HTML,
                styles: [VULNERABILITY_CONFIG_STYLES, REGISTRY_CONFIG_STYLES]
            },] },
];
/** @nocollapse */
VulnerabilityConfigComponent.ctorParameters = function () { return [
    { type: ScanningResultService, },
    { type: ErrorHandler, },
    { type: TranslateService, },
    { type: SystemInfoService, },
]; };
VulnerabilityConfigComponent.propDecorators = {
    'configChange': [{ type: Output },],
    'vulnerabilityConfig': [{ type: Input },],
    'showSubTitle': [{ type: Input },],
    'showScanningNamespaces': [{ type: Input },],
    'systemSettingsForm': [{ type: ViewChild, args: ["systemConfigFrom",] },],
};
//# sourceMappingURL=vulnerability-config.component.js.map