import { Component, Input, ChangeDetectorRef, ViewChild } from '@angular/core';
import { SCANNING_STYLES } from './scanning.css';
import { BAR_CHART_COMPONENT_HTML } from './scanning.html';
import { VULNERABILITY_SCAN_STATUS } from '../utils';
import { TagService, ScanningResultService } from '../service/index';
import { ErrorHandler } from '../error-handler/index';
import { toPromise } from '../utils';
import { Observable } from 'rxjs/Rx';
import { ChannelService } from '../channel/index';
var STATE_CHECK_INTERVAL = 2000; //2s
var RETRY_TIMES = 3;
var ResultBarChartComponent = (function () {
    function ResultBarChartComponent(tagService, scanningService, errorHandler, channel, ref) {
        this.tagService = tagService;
        this.scanningService = scanningService;
        this.errorHandler = errorHandler;
        this.channel = channel;
        this.ref = ref;
        this.repoName = "";
        this.tagId = "";
        this.onSubmitting = false;
        this.retryCounter = 0;
    }
    ResultBarChartComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.scanSubscription = this.channel.scanCommand$.subscribe(function (tagId) {
            var myFullTag = _this.repoName + "/" + _this.tagId;
            if (myFullTag === tagId) {
                _this.scanNow();
            }
        });
    };
    ResultBarChartComponent.prototype.ngOnDestroy = function () {
        if (this.stateCheckTimer) {
            this.stateCheckTimer.unsubscribe();
            this.stateCheckTimer = null;
        }
        if (this.scanSubscription) {
            this.scanSubscription.unsubscribe();
        }
        ;
    };
    Object.defineProperty(ResultBarChartComponent.prototype, "status", {
        //Get vulnerability scanning status 
        get: function () {
            if (this.summary && this.summary.scan_status) {
                return this.summary.scan_status;
            }
            return VULNERABILITY_SCAN_STATUS.stopped;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "completed", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.finished;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "error", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.error;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "queued", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.pending;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "scanning", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.running;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "stopped", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.stopped;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResultBarChartComponent.prototype, "unknown", {
        get: function () {
            return this.status === VULNERABILITY_SCAN_STATUS.unknown;
        },
        enumerable: true,
        configurable: true
    });
    ResultBarChartComponent.prototype.scanNow = function () {
        var _this = this;
        if (this.onSubmitting) {
            //Avoid duplicated submitting
            return;
        }
        if (!this.repoName || !this.tagId) {
            return;
        }
        this.onSubmitting = true;
        toPromise(this.scanningService.startVulnerabilityScanning(this.repoName, this.tagId))
            .then(function () {
            _this.onSubmitting = false;
            //Forcely change status to queued after successful submitting
            _this.summary.scan_status = VULNERABILITY_SCAN_STATUS.pending;
            //Forcely refresh view 
            _this.forceRefreshView(1000);
            //Start check status util the job is done
            if (!_this.stateCheckTimer) {
                //Avoid duplicated subscribing
                _this.stateCheckTimer = Observable.timer(STATE_CHECK_INTERVAL, STATE_CHECK_INTERVAL).subscribe(function () {
                    _this.getSummary();
                });
            }
        })
            .catch(function (error) {
            _this.onSubmitting = false;
            _this.errorHandler.error(error);
        });
    };
    ResultBarChartComponent.prototype.getSummary = function () {
        var _this = this;
        if (!this.repoName || !this.tagId) {
            return;
        }
        toPromise(this.tagService.getTag(this.repoName, this.tagId))
            .then(function (t) {
            //To keep the same summary reference, use value copy.
            _this.copyValue(t.scan_overview);
            //Forcely refresh view 
            _this.forceRefreshView(1000);
            if (!_this.queued && !_this.scanning) {
                //Scanning should be done
                if (_this.stateCheckTimer) {
                    _this.stateCheckTimer.unsubscribe();
                    _this.stateCheckTimer = null;
                }
            }
        })
            .catch(function (error) {
            _this.errorHandler.error(error);
            _this.retryCounter++;
            if (_this.retryCounter >= RETRY_TIMES) {
                //Stop timer
                if (_this.stateCheckTimer) {
                    _this.stateCheckTimer.unsubscribe();
                    _this.stateCheckTimer = null;
                }
                _this.retryCounter = 0;
            }
        });
    };
    ResultBarChartComponent.prototype.copyValue = function (newVal) {
        if (!newVal || !newVal.scan_status) {
            return;
        }
        this.summary.scan_status = newVal.scan_status;
        this.summary.job_id = newVal.job_id;
        this.summary.severity = newVal.severity;
        this.summary.components = newVal.components;
        this.summary.update_time = newVal.update_time;
    };
    ResultBarChartComponent.prototype.forceRefreshView = function (duration) {
        var _this = this;
        //Reset timer
        if (this.timerHandler) {
            clearInterval(this.timerHandler);
        }
        this.timerHandler = setInterval(function () { return _this.ref.markForCheck(); }, 100);
        setTimeout(function () {
            if (_this.timerHandler) {
                clearInterval(_this.timerHandler);
                _this.timerHandler = null;
            }
        }, duration);
    };
    //Check error log
    ResultBarChartComponent.prototype.viewLog = function () {
        if (this.summary && this.summary.job_id) {
            this.scanningJobLogViewer.open(this.summary.job_id);
        }
    };
    return ResultBarChartComponent;
}());
export { ResultBarChartComponent };
ResultBarChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'hbr-vulnerability-bar',
                styles: [SCANNING_STYLES],
                template: BAR_CHART_COMPONENT_HTML
            },] },
];
/** @nocollapse */
ResultBarChartComponent.ctorParameters = function () { return [
    { type: TagService, },
    { type: ScanningResultService, },
    { type: ErrorHandler, },
    { type: ChannelService, },
    { type: ChangeDetectorRef, },
]; };
ResultBarChartComponent.propDecorators = {
    'repoName': [{ type: Input },],
    'tagId': [{ type: Input },],
    'summary': [{ type: Input },],
    'scanningJobLogViewer': [{ type: ViewChild, args: ["scanningLogViewer",] },],
};
//# sourceMappingURL=result-bar-chart.component.js.map